@@ -13,23 +13,7 @@
 # Copyright (c) 2008, Andrey Kiselev <dron16@ak4719.spb.edu>
 # Copyright (c) 2008-2014, Even Rouault <even dot rouault at spatialys.com>
 #
-# Permission is hereby granted, free of charge, to any person obtaining a
-# copy of this software and associated documentation files (the "Software"),
-# to deal in the Software without restriction, including without limitation
-# the rights to use, copy, modify, merge, publish, distribute, sublicense,
-# and/or sell copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included
-# in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-# DEALINGS IN THE SOFTWARE.
+# SPDX-License-Identifier: MIT
 ###############################################################################
 
 import math
@@ -40,6 +24,11 @@ import struct
 import gdaltest
 import pytest
 
+pytestmark = pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
+
 from osgeo import gdal, osr
 
 ###############################################################################
@@ -280,6 +269,17 @@ def test_warp_5_downsize():
     assert maxdiff <= 1, "Image too different from reference"
 
 
+def test_warp_lanczos_downsize_50_75():
+
+    ds = gdal.Open("data/utmsmall_lanczos_50_75.vrt")
+    ref_ds = gdal.Open("data/utmsmall_lanczos_50_75.tif")
+    maxdiff = gdaltest.compare_ds(ds, ref_ds)
+    ds = None
+    ref_ds = None
+
+    assert maxdiff <= 1, "Image too different from reference"
+
+
 # Downsampling
 
 
@@ -1409,7 +1409,7 @@ def test_warp_52():
     assert end - start <= 10, "processing time was way too long"
 
     cs = out_ds.GetRasterBand(4).Checksum()
-    assert cs == 3188
+    assert cs == 3177
 
 
 ###############################################################################
