@@ -8,23 +8,7 @@
  ******************************************************************************
  * Copyright (c) 2018, Hugo Mercier, <hugo dot mercier at oslandia dot com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "gdal_unit_test.h"
@@ -40,6 +24,8 @@
 #include <fstream>
 #endif
 
+#include <limits>
+
 #include "gtest_include.h"
 
 namespace marching_squares
@@ -171,7 +157,8 @@ TEST_F(test_ms_polygon, dummy)
     TestPolygonWriter w;
     {
         PolygonRingAppender<TestPolygonWriter> appender(w);
-        IntervalLevelRangeIterator levels(0.0, 10.0);
+        IntervalLevelRangeIterator levels(
+            0.0, 10.0, -std::numeric_limits<double>::infinity());
         SegmentMerger<PolygonRingAppender<TestPolygonWriter>,
                       IntervalLevelRangeIterator>
             writer(appender, levels, /* polygonize */ true);
@@ -234,7 +221,8 @@ TEST_F(test_ms_polygon, four_pixels)
     TestPolygonWriter w;
     {
         PolygonRingAppender<TestPolygonWriter> appender(w);
-        IntervalLevelRangeIterator levels(0.0, 10.0);
+        IntervalLevelRangeIterator levels(
+            0.0, 10.0, -std::numeric_limits<double>::infinity());
         SegmentMerger<PolygonRingAppender<TestPolygonWriter>,
                       IntervalLevelRangeIterator>
             writer(appender, levels, /* polygonize */ true);
@@ -304,7 +292,9 @@ TEST_F(test_ms_polygon, four_pixels_2)
     {
         PolygonRingAppender<TestPolygonWriter> appender(w);
         const double levels[] = {155.0};
-        FixedLevelRangeIterator levelGenerator(levels, 1);
+        FixedLevelRangeIterator levelGenerator(
+            levels, 1, -std::numeric_limits<double>::infinity(),
+            std::numeric_limits<double>::infinity());
         SegmentMerger<PolygonRingAppender<TestPolygonWriter>,
                       FixedLevelRangeIterator>
             writer(appender, levelGenerator, /* polygonize */ true);
@@ -400,7 +390,8 @@ TEST_F(test_ms_polygon, nine_pixels)
     TestPolygonWriter w;
     {
         PolygonRingAppender<TestPolygonWriter> appender(w);
-        IntervalLevelRangeIterator levels(1.0, 10.0);
+        IntervalLevelRangeIterator levels(
+            1.0, 10.0, -std::numeric_limits<double>::infinity());
         SegmentMerger<PolygonRingAppender<TestPolygonWriter>,
                       IntervalLevelRangeIterator>
             writer(appender, levels, /* polygonize */ true);
@@ -452,7 +443,8 @@ TEST_F(test_ms_polygon, three_nested_rings)
     TestPolygonWriter w;
     {
         PolygonRingAppender<TestPolygonWriter> appender(w);
-        IntervalLevelRangeIterator levels(1.0, 2.0);
+        IntervalLevelRangeIterator levels(
+            1.0, 2.0, -std::numeric_limits<double>::infinity());
         SegmentMerger<PolygonRingAppender<TestPolygonWriter>,
                       IntervalLevelRangeIterator>
             writer(appender, levels, /* polygonize */ true);
