@@ -18,6 +18,7 @@
 #include <stdlib.h>
 #include <math.h>
 
+#include <cmath>
 #include <limits>
 
 #include "degrib2.h"
@@ -29,6 +30,7 @@
 #include "metaname.h"
 #include "clock.h"
 #include "cpl_error.h"
+#include "cpl_string.h"
 
 /* default missing data value (see: bitmap GRIB1: sect3 and sect4) */
 /* UNDEFINED is default, UNDEFINED_PRIM is desired choice. */
@@ -169,6 +171,7 @@ static const GRIB1ParmTable *Choose_ParmTable (pdsG1Type *pdsMeta,
                if ((subcenter != 0) || ((process != 80) && (process != 180))) {
                   return &parm_table_ncep_opn[0];
                }
+#if 0
                /* At this point could be either the opn or reanalysis table */
                switch (DEF_NCEP_TABLE) {
                   case opn_nowarn:
@@ -177,6 +180,10 @@ static const GRIB1ParmTable *Choose_ParmTable (pdsG1Type *pdsMeta,
                      return &parm_table_ncep_reanal[0];
                }
                break;
+#else
+               // ERO: this is the non convoluted version of the above code
+               return &parm_table_ncep_reanal[0];
+#endif
             case 3:
                return &parm_table_ncep_opn[0];
             case 128:
@@ -234,6 +241,8 @@ static const GRIB1ParmTable *Choose_ParmTable (pdsG1Type *pdsMeta,
                return &parm_table_ecmwf_170[0];
             case 180:
                return &parm_table_ecmwf_180[0];
+            case 228:
+               return &parm_table_ecmwf_228[0];
          }
          break;
       case DWD:
@@ -277,16 +286,16 @@ static const GRIB1ParmTable *Choose_ParmTable (pdsG1Type *pdsMeta,
          break;
    }
    if (pdsMeta->mstrVersion > 3) {
-      CPLDebug( "GRIB", "Don't understand the parameter table, since center %d-%d used\n"
+      CPLError( CE_Warning, CPLE_AppDefined, "GRIB: Don't understand the parameter table, since center %d-%d used\n"
               "parameter table version %d instead of 3 (international exchange).\n"
-              "Using default for now, but please email arthur.taylor@noaa.gov\n"
+              "Using default for now (which might lead to erroneous interpretation), but please email arthur.taylor@noaa.gov\n"
               "about adding this table to his 'degrib1.c' and 'grib1tab.c' files.",
               center, subcenter, pdsMeta->mstrVersion);
    }
    if (pdsMeta->cat > 127) {
-      CPLDebug( "GRIB", "Parameter %d is > 127, so it falls in the local use section of\n"
+      CPLError(CE_Warning, CPLE_AppDefined, "GRIB: Parameter %d is > 127, so it falls in the local use section of\n"
               "the parameter table (and is undefined on the international table.\n"
-              "Using default for now, but please email arthur.taylor@noaa.gov\n"
+              "Using default for now(which might lead to erroneous interpretation), but please email arthur.taylor@noaa.gov\n"
               "about adding this table to his 'degrib1.c' and 'grib1tab.c' files.",
               pdsMeta->cat);
    }
@@ -340,6 +349,13 @@ static void GRIB1_Table2LookUp (pdsG1Type *pdsMeta, const char **name,
       }
    }
    *name = table[pdsMeta->cat].name;
+   if( strcmp(*name, CPLSPrintf("var%d", pdsMeta->cat)) == 0 )
+   {
+       if( center == ECMWF )
+           *name = CPLSPrintf("var%d of table %d of center ECMWF", pdsMeta->cat, pdsMeta->mstrVersion);
+       else
+           *name = CPLSPrintf("var%d of table %d of center %d", pdsMeta->cat, pdsMeta->mstrVersion, center);
+   }
    *comment = table[pdsMeta->cat].comment;
    *unit = table[pdsMeta->cat].unit;
    *convert = table[pdsMeta->cat].convert;
@@ -710,7 +726,7 @@ static int ReadGrib1Sect1 (uChar *pds, uInt4 pdsLen, uInt4 gribLen, uInt4 *curLo
  * NOTES
  *****************************************************************************
  */
-int GRIB1_Inventory (DataSource &fp, uInt4 gribLen, inventoryType *inv)
+int GRIB1_Inventory (VSILFILE *fp, uInt4 gribLen, inventoryType *inv)
 {
    uChar temp[3];        /* Used to determine the section length. */
    uInt4 sectLen;       /* Length in bytes of the current section. */
@@ -729,7 +745,7 @@ int GRIB1_Inventory (DataSource &fp, uInt4 gribLen, inventoryType *inv)
    unsigned short int subcenter; /* The Sub Center that created the data */
 
    curLoc = 8;
-   if (fp.DataSourceFread(temp, sizeof (char), 3) != 3) {
+   if (VSIFReadL(temp, sizeof (char), 3, fp) != 3) {
       errSprintf ("Ran out of file.\n");
       return -1;
    }
@@ -752,7 +768,7 @@ int GRIB1_Inventory (DataSource &fp, uInt4 gribLen, inventoryType *inv)
    *pds = *temp;
    pds[1] = temp[1];
    pds[2] = temp[2];
-   if (fp.DataSourceFread(pds + 3, sizeof (char), sectLen - 3) + 3 != sectLen) {
+   if (VSIFReadL(pds + 3, sizeof (char), sectLen - 3, fp) + 3 != sectLen) {
       errSprintf ("Ran out of file.\n");
       free (pds);
       return -1;
@@ -793,7 +809,7 @@ int GRIB1_Inventory (DataSource &fp, uInt4 gribLen, inventoryType *inv)
    return 0;
 }
 
-int GRIB1_RefTime (DataSource &fp, uInt4 gribLen, double *refTime)
+int GRIB1_RefTime (VSILFILE *fp, uInt4 gribLen, double *refTime)
 {
    uChar temp[3];        /* Used to determine the section length. */
    uInt4 sectLen;       /* Length in bytes of the current section. */
@@ -808,7 +824,7 @@ int GRIB1_RefTime (DataSource &fp, uInt4 gribLen, double *refTime)
    unsigned short int subcenter; /* The Sub Center that created the data */
 
    curLoc = 8;
-   if (fp.DataSourceFread (temp, sizeof (char), 3) != 3) {
+   if (VSIFReadL (temp, sizeof (char), 3, fp) != 3) {
       errSprintf ("Ran out of file.\n");
       return -1;
    }
@@ -821,7 +837,7 @@ int GRIB1_RefTime (DataSource &fp, uInt4 gribLen, double *refTime)
    *pds = *temp;
    pds[1] = temp[1];
    pds[2] = temp[2];
-   if (fp.DataSourceFread (pds + 3, sizeof (char), sectLen - 3) + 3 != sectLen) {
+   if (VSIFReadL (pds + 3, sizeof (char), sectLen - 3, fp) + 3 != sectLen) {
       errSprintf ("Ran out of file.\n");
       free (pds);
       return -1;
@@ -1133,6 +1149,7 @@ static int ReadGrib1Sect2 (uChar *gds, uInt4 gribLen, uInt4 *curLoc,
          if (gdsMeta->center & GRIB2BIT_1) {
             /* South polar stereographic. */
             gdsMeta->scaleLat1 = gdsMeta->scaleLat2 = -90;
+            gdsMeta->meshLat = -gdsMeta->meshLat;
          } else {
             /* North polar stereographic. */
             gdsMeta->scaleLat1 = gdsMeta->scaleLat2 = 90;
@@ -1262,7 +1279,7 @@ printf ("Nx = %ld, Ny = %ld\n", gdsMeta->Nx, gdsMeta->Ny);
  *     bms = The compressed part of the message dealing with "BMS". (Input)
  * gribLen = The total length of the GRIB1 message. (Input)
  *  curLoc = Current location in the GRIB1 message. (Output)
- *  bitmap = The extracted bitmap. (Output)
+ * pBitmap = Pointer to the extracted bitmap. (Output)
  *    NxNy = The total size of the grid. (Input)
  *
  * FILES/DATABASES: None
@@ -1279,13 +1296,15 @@ printf ("Nx = %ld, Ny = %ld\n", gdsMeta->Nx, gdsMeta->Ny);
  *****************************************************************************
  */
 static int ReadGrib1Sect3 (uChar *bms, uInt4 gribLen, uInt4 *curLoc,
-                           uChar *bitmap, uInt4 NxNy)
+                           uChar **pBitmap, uInt4 NxNy)
 {
    uInt4 sectLen;       /* Length in bytes of the current section. */
    short int numeric;   /* Determine if this is a predefined bitmap */
    uChar bits;          /* Used to locate which bit we are currently using. */
    uInt4 i;             /* Helps traverse the bitmap. */
 
+   *pBitmap = nullptr;
+
    uInt4 bmsRemainingSize = gribLen - *curLoc;
    if( bmsRemainingSize < 6 )
    {
@@ -1306,8 +1325,9 @@ static int ReadGrib1Sect3 (uChar *bms, uInt4 gribLen, uInt4 *curLoc,
    bms += 3;
    /* Assert: *bms currently points to number of unused bits at end of BMS. */
    if (NxNy + *bms + 6 * 8 != sectLen * 8) {
-      errSprintf ("NxNy + # of unused bits %ld != # of available bits %ld\n",
-                  (sInt4) (NxNy + *bms), (sInt4) ((sectLen - 6) * 8));
+      errSprintf ("NxNy + # of unused bits != # of available bits\n");
+      // commented out to avoid unsigned integer overflow
+      // (sInt4) (NxNy + *bms), (sInt4) ((sectLen - 6) * 8));
       return -2;
    }
    bms++;
@@ -1324,6 +1344,13 @@ static int ReadGrib1Sect3 (uChar *bms, uInt4 gribLen, uInt4 *curLoc,
       errSprintf ("Ran out of data in BMS (GRIB 1 Section 3)\n");
       return -1;
    }
+   *pBitmap = (uChar*) malloc(NxNy);
+   auto bitmap = *pBitmap;
+   if( bitmap== nullptr )
+   {
+      errSprintf ("Ran out of memory in allocating bitmap (GRIB 1 Section 3)\n");
+      return -1;
+   }
    bits = 0x80;
    for (i = 0; i < NxNy; i++) {
       *(bitmap++) = (*bms) & bits;
@@ -1606,8 +1633,15 @@ static int ReadGrib1Sect4 (uChar *bds, uInt4 gribLen, uInt4 *curLoc,
 #endif
    }
 
-   if (!f_bms && (meta->gds.numPts * numBits + numUnusedBit) !=
-       (sectLen - 11) * 8) {
+   if (!f_bms && (
+       sectLen < 11 ||
+       (numBits > 0 && meta->gds.numPts > UINT_MAX / numBits) ||
+       (meta->gds.numPts * numBits > UINT_MAX - numUnusedBit))) {
+     printf ("numPts * (numBits in a Group) + # of unused bits != "
+              "# of available bits\n");
+   }
+   else if (!f_bms &&
+            (meta->gds.numPts * numBits + numUnusedBit) != (sectLen - 11) * 8) {
       printf ("numPts * (numBits in a Group) + # of unused bits %d != "
               "# of available bits %d\n",
               (sInt4) (meta->gds.numPts * numBits + numUnusedBit),
@@ -1647,7 +1681,7 @@ static int ReadGrib1Sect4 (uChar *bds, uInt4 gribLen, uInt4 *curLoc,
    meta->gridAttrib.max = meta->gridAttrib.min;
    meta->gridAttrib.f_maxmin = 1;
    meta->gridAttrib.numMiss = 0;
-   if (refVal >= std::numeric_limits<float>::max() || CPLIsNan(refVal)) {
+   if (refVal >= std::numeric_limits<float>::max() || std::isnan(refVal)) {
       meta->gridAttrib.refVal = std::numeric_limits<float>::max();
    } else if (refVal <= -std::numeric_limits<float>::max()) {
       meta->gridAttrib.refVal = -std::numeric_limits<float>::max();
@@ -1661,6 +1695,15 @@ static int ReadGrib1Sect4 (uChar *bds, uInt4 gribLen, uInt4 *curLoc,
     * can convert it. */
    f_convert = ((meta->gds.scan & 0xe0) != 0x40);
 
+   if (f_bms) {
+      meta->gridAttrib.f_miss = 1;
+      meta->gridAttrib.missPri = UNDEFINED;
+   }
+   else
+   {
+      meta->gridAttrib.f_miss = 0;
+   }
+
    if (f_bms) {
 /*
 #ifdef DEBUG
@@ -1668,8 +1711,6 @@ static int ReadGrib1Sect4 (uChar *bds, uInt4 gribLen, uInt4 *curLoc,
 #endif
 */
       /* Start unpacking the data, assuming there is a bitmap. */
-      meta->gridAttrib.f_miss = 1;
-      meta->gridAttrib.missPri = UNDEFINED;
       for (i = 0; i < meta->gds.numPts; i++) {
          /* Find the destination index. */
          if (f_convert) {
@@ -1684,7 +1725,7 @@ static int ReadGrib1Sect4 (uChar *bds, uInt4 gribLen, uInt4 *curLoc,
          // cppcheck-suppress nullPointer
          if (!bitmap[i]) {
             meta->gridAttrib.numMiss++;
-            data[newIndex] = UNDEFINED;
+            if( data ) data[newIndex] = UNDEFINED;
          } else {
             if (numBits != 0) {
                 if( ((int)bdsRemainingSize - 1) * 8 + bufLoc < (int)numBits )
@@ -1704,11 +1745,11 @@ static int ReadGrib1Sect4 (uChar *bds, uInt4 gribLen, uInt4 *curLoc,
                if (meta->gridAttrib.max < d_temp) {
                   meta->gridAttrib.max = d_temp;
                }
-               data[newIndex] = d_temp;
+               if( data ) data[newIndex] = d_temp;
             } else {
                /* Assert: d_temp = unitM * refVal / pow (10.0,DSF) + unitB. */
                /* Assert: min = unitM * refVal / pow (10.0, DSF) + unitB. */
-               data[newIndex] = meta->gridAttrib.min;
+               if( data ) data[newIndex] = meta->gridAttrib.min;
             }
          }
       }
@@ -1725,6 +1766,8 @@ static int ReadGrib1Sect4 (uChar *bds, uInt4 gribLen, uInt4 *curLoc,
       }
       if (resetPrim != 0) {
          meta->gridAttrib.missPri = resetPrim;
+      }
+      if (data != nullptr && resetPrim != 0) {
          for (i = 0; i < meta->gds.numPts; i++) {
             /* Find the destination index. */
             if (f_convert) {
@@ -1744,6 +1787,9 @@ static int ReadGrib1Sect4 (uChar *bds, uInt4 gribLen, uInt4 *curLoc,
 
    } else {
 
+      if( !data )
+        return 0;
+
 #ifdef DEBUG
 /*
       printf ("There is no bitmap?\n");
@@ -1751,7 +1797,6 @@ static int ReadGrib1Sect4 (uChar *bds, uInt4 gribLen, uInt4 *curLoc,
 #endif
 
       /* Start unpacking the data, assuming there is NO bitmap. */
-      meta->gridAttrib.f_miss = 0;
       for (i = 0; i < meta->gds.numPts; i++) {
          if (numBits != 0) {
             /* Find the destination index. */
@@ -1852,7 +1897,7 @@ static int ReadGrib1Sect4 (uChar *bds, uInt4 gribLen, uInt4 *curLoc,
  * 3) Should add unitM / unitB support.
  *****************************************************************************
  */
-int ReadGrib1Record (DataSource &fp, sChar f_unit, double **Grib_Data,
+int ReadGrib1Record (VSILFILE *fp, sChar f_unit, double **Grib_Data,
                      uInt4 *grib_DataLen, grib_MetaData *meta,
                      IS_dataType *IS, sInt4 sect0[SECT0LEN_WORD],
                      uInt4 gribLen, double majEarth, double minEarth)
@@ -1863,7 +1908,7 @@ int ReadGrib1Record (DataSource &fp, sChar f_unit, double **Grib_Data,
    uInt4 curLoc;        /* Current location in the GRIB message. */
    char f_gds;          /* flag if there is a gds section. */
    char f_bms;          /* flag if there is a bms section. */
-   double *grib_Data;   /* A pointer to Grib_Data for ease of manipulation. */
+   double *grib_Data = nullptr;   /* A pointer to Grib_Data for ease of manipulation. */
    uChar *bitmap = nullptr; /* A char field (0=noData, 1=data) set up in BMS. */
    short int DSF;       /* Decimal Scale Factor for unpacking the data. */
    double unitM = 1;    /* M in y = Mx + B, for unit conversion. */
@@ -1880,9 +1925,27 @@ int ReadGrib1Record (DataSource &fp, sChar f_unit, double **Grib_Data,
    /* nd5 needs to be gribLen in (sInt4) units rounded up. */
    nd5 = (gribLen + 3) / 4;
    if (nd5 > IS->ipackLen) {
+      if( gribLen > 100 * 1024 * 1024 )
+      {
+         vsi_l_offset curPos = VSIFTellL(fp);
+         VSIFSeekL(fp, 0, SEEK_END);
+         vsi_l_offset fileSize = VSIFTellL(fp);
+         VSIFSeekL(fp, curPos, SEEK_SET);
+         if( fileSize < gribLen )
+         {
+            errSprintf("File too short");
+            return -1;
+         }
+      }
+      sInt4* newipack = (sInt4 *) realloc ((void *) (IS->ipack),
+                                     nd5 * sizeof (sInt4));
+      if( newipack == nullptr )
+      {
+          errSprintf ("Out of memory\n");
+          return -1;
+      }
       IS->ipackLen = nd5;
-      IS->ipack = (sInt4 *) realloc ((void *) (IS->ipack),
-                                     (IS->ipackLen) * sizeof (sInt4));
+      IS->ipack = newipack;
    }
    c_ipack = (uChar *) IS->ipack;
    /* Init last sInt4 to 0, to make sure that the padded bytes are 0. */
@@ -1890,8 +1953,8 @@ int ReadGrib1Record (DataSource &fp, sChar f_unit, double **Grib_Data,
    /* Init first 2 sInt4 to sect0. */
    memcpy (c_ipack, sect0, SECT0LEN_WORD * 2);
    /* Read in the rest of the message. */
-   if (fp.DataSourceFread (c_ipack + SECT0LEN_WORD * 2, sizeof (char),
-              (gribLen - SECT0LEN_WORD * 2)) + SECT0LEN_WORD * 2 != gribLen) {
+   if (VSIFReadL (c_ipack + SECT0LEN_WORD * 2, sizeof (char),
+              (gribLen - SECT0LEN_WORD * 2), fp) + SECT0LEN_WORD * 2 != gribLen) {
       errSprintf ("Ran out of file\n");
       return -1;
    }
@@ -1941,14 +2004,16 @@ int ReadGrib1Record (DataSource &fp, sChar f_unit, double **Grib_Data,
       }
    }
 
-   /* Allocate memory for the grid. */
-   if (meta->gds.numPts > *grib_DataLen) {
+   if( Grib_Data )
+   {
+     /* Allocate memory for the grid. */
+     if (meta->gds.numPts > *grib_DataLen) {
       if( meta->gds.numPts > 100 * 1024 * 1024 )
       {
-          long curPos = fp.DataSourceFtell();
-          fp.DataSourceFseek(0, SEEK_END);
-          long fileSize = fp.DataSourceFtell();
-          fp.DataSourceFseek(curPos, SEEK_SET);
+          vsi_l_offset curPos = VSIFTellL(fp);
+          VSIFSeekL(fp, 0, SEEK_END);
+          vsi_l_offset fileSize = VSIFTellL(fp);
+          VSIFSeekL(fp, curPos, SEEK_SET);
           // allow a compression ratio of 1:1000
           if( meta->gds.numPts / 1000 > (uInt4)fileSize )
           {
@@ -1959,6 +2024,16 @@ int ReadGrib1Record (DataSource &fp, sChar f_unit, double **Grib_Data,
           }
       }
 
+#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
+      if( meta->gds.numPts > static_cast<size_t>(INT_MAX) /  sizeof (double) )
+      {
+          errSprintf ("Memory allocation failed due to being bigger than 2 GB in fuzzing mode");
+          *grib_DataLen = 0;
+          *Grib_Data = nullptr;
+          return -2;
+      }
+#endif
+
       *grib_DataLen = meta->gds.numPts;
       *Grib_Data = (double *) realloc ((void *) (*Grib_Data),
                                        (*grib_DataLen) * sizeof (double));
@@ -1967,13 +2042,13 @@ int ReadGrib1Record (DataSource &fp, sChar f_unit, double **Grib_Data,
         *grib_DataLen = 0;
         return -1;
       }
+     }
+     grib_Data = *Grib_Data;
    }
-   grib_Data = *Grib_Data;
 
    /* Get the Bit Map Section. */
    if (f_bms) {
-      bitmap = (uChar *) malloc (meta->gds.numPts * sizeof (uChar));
-      if (ReadGrib1Sect3 (c_ipack + curLoc, gribLen, &curLoc, bitmap,
+      if (ReadGrib1Sect3 (c_ipack + curLoc, gribLen, &curLoc, &bitmap,
                           meta->gds.numPts) != 0) {
          free (bitmap);
          preErrSprintf ("Inside ReadGrib1Record\n");
@@ -2101,49 +2176,58 @@ int ReadGrib1Record (DataSource &fp, sChar f_unit, double **Grib_Data,
 #ifdef DEBUG_DEGRIB1
 int main (int argc, char **argv)
 {
-   DataSource grib_fp;       /* The opened grib2 file for input. */
-   sInt4 offset;        /* Where we currently are in grib_fp. */
-   sInt4 sect0[SECT0LEN_WORD]; /* Holds the current Section 0. */
-   char wmo[WMO_HEADER_LEN + 1]; /* Holds the current wmo message. */
-   sInt4 gribLen;       /* Length of the current GRIB message. */
-   sInt4 wmoLen;        /* Length of current wmo Message. */
+   VSILFILE * grib_fp;       /* The opened grib2 file for input. */
+   char *buff = nullptr;
+   uInt4 buffLen = 0;
+   sInt4 sect0[SECT0LEN_WORD] = { 0 }; /* Holds the current Section 0. */
+   uInt4 gribLen;       /* Length of the current GRIB message. */
    char *msg;
    int version;
    sChar f_unit = 0;
    double *grib_Data;
-   sInt4 grib_DataLen;
+   uInt4 grib_DataLen;
    grib_MetaData meta;
+
+   double majEarth = 0.0;  // -radEarth if < 6000 ignore, otherwise use this
+                           // to override the radEarth in the GRIB1 or GRIB2
+                           // message.  Needed because NCEP uses 6371.2 but
+                           // GRIB1 could only state 6367.47.
+   double minEarth = 0.0;  // -minEarth if < 6000 ignore, otherwise use this
+                           // to override the minEarth in the GRIB1 or GRIB2
+                           // message.
+
+
    IS_dataType is;      /* Un-parsed meta data for this GRIB2 message. As
                          * well as some memory used by the unpacker. */
 
-   //if ((grib_fp = fopen (argv[1], "rb")) == NULL) {
-   //   printf ("Problems opening %s for read\n", argv[1]);
-   //   return 1;
-   //}
-	 grib_fp = FileDataSource(argv[1]);
+   if ((grib_fp = VSIFOpenL (argv[1], "rb")) == NULL) {
+      printf ("Problems opening %s for read\n", argv[1]);
+      return 1;
+   }
    IS_Init (&is);
    MetaInit (&meta);
 
-   offset = 0;
-   if (ReadSECT0 (grib_fp, offset, WMO_HEADER_LEN, WMO_SECOND_LEN, wmo,
-                  sect0, &gribLen, &wmoLen, &version) < 0) {
+   if (ReadSECT0 (grib_fp, &buff, &buffLen, -1, sect0, &gribLen, &version) < 0) {
+      VSIFCloseL(grib_fp);
+      free(buff);
       msg = errSprintf (NULL);
       printf ("%s\n", msg);
       return -1;
    }
+   free(buff);
+
    grib_DataLen = 0;
    grib_Data = NULL;
    if (version == 1) {
       meta.GribVersion = version;
       ReadGrib1Record (grib_fp, f_unit, &grib_Data, &grib_DataLen, &meta,
-                       &is, sect0, gribLen);
-      offset = offset + gribLen + wmoLen;
+                       &is, sect0, gribLen, majEarth, minEarth);
    }
 
    MetaFree (&meta);
    IS_Free (&is);
    free (grib_Data);
-   //fclose (grib_fp);
+   VSIFCloseL(grib_fp);
    return 0;
 }
 #endif
