@@ -10,23 +10,7 @@
 ###############################################################################
 # Copyright (c) 2008-2013, Even Rouault <even dot rouault at spatialys.com>
 #
-# Permission is hereby granted, free of charge, to any person obtaining a
-# copy of this software and associated documentation files (the "Software"),
-# to deal in the Software without restriction, including without limitation
-# the rights to use, copy, modify, merge, publish, distribute, sublicense,
-# and/or sell copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included
-# in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-# DEALINGS IN THE SOFTWARE.
+# SPDX-License-Identifier: MIT
 ###############################################################################
 
 import os
@@ -68,6 +52,15 @@ def test_basic_test_1():
     pytest.fail("did not get expected error message, got %s" % gdal.GetLastErrorMsg())
 
 
+def test_basic_test_invalid_open_flag():
+    with pytest.raises(Exception, match="invalid value for GDALAccess"):
+        gdal.Open("data/byte.tif", "invalid")
+
+    assert gdal.OF_RASTER not in (gdal.GA_ReadOnly, gdal.GA_Update)
+    with pytest.raises(Exception, match="invalid value for GDALAccess"):
+        gdal.Open("data/byte.tif", gdal.OF_RASTER)
+
+
 @pytest.mark.skipif(sys.platform != "linux", reason="Incorrect platform")
 def test_basic_test_strace_non_existing_file():
 
@@ -331,6 +324,22 @@ def test_basic_test_11():
         with pytest.raises(Exception):
             gdal.OpenEx("non existing")
 
+    try:
+        with gdal.ExceptionMgr(useExceptions=True):
+            try:
+                gdal.OpenEx("non existing")
+            except Exception:
+                pass
+    except Exception:
+        pytest.fails("Exception thrown whereas it should not have")
+
+    with gdal.ExceptionMgr(useExceptions=True):
+        try:
+            gdal.OpenEx("non existing")
+        except Exception:
+            pass
+        gdal.Open("data/byte.tif")
+
 
 ###############################################################################
 # Test GDAL layer API
@@ -938,3 +947,40 @@ def test_tmp_vsimem(tmp_vsimem):
     assert isinstance(tmp_vsimem, os.PathLike)
 
     assert gdal.VSIStatL(tmp_vsimem) is not None
+
+
+def test_band_iter():
+
+    ds = gdal.Open("data/rgba.tif")
+
+    assert len(ds) == 4
+
+    bands = []
+
+    for band in ds:
+        bands.append(band)
+
+    assert len(bands) == 4
+
+
+def test_band_getitem():
+
+    ds = gdal.Open("data/rgba.tif")
+
+    assert ds[2].this == ds.GetRasterBand(2).this
+
+    with pytest.raises(IndexError):
+        ds[0]
+
+    with pytest.raises(IndexError):
+        ds[5]
+
+
+def test_colorinterp():
+
+    d = {}
+    for c in range(gdal.GCI_Max + 1):
+        name = gdal.GetColorInterpretationName(c)
+        assert name not in d
+        d[name] = c
+        assert gdal.GetColorInterpretationByName(name) == c
