@@ -10,23 +10,7 @@
 # Copyright (c) 2007, Frank Warmerdam <warmerdam@pobox.com>
 # Copyright (c) 2008-2012, Even Rouault <even dot rouault at spatialys.com>
 #
-# Permission is hereby granted, free of charge, to any person obtaining a
-# copy of this software and associated documentation files (the "Software"),
-# to deal in the Software without restriction, including without limitation
-# the rights to use, copy, modify, merge, publish, distribute, sublicense,
-# and/or sell copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included
-# in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-# DEALINGS IN THE SOFTWARE.
+# SPDX-License-Identifier: MIT
 ###############################################################################
 
 import os
@@ -72,6 +56,10 @@ def test_mask_1():
 # Verify the checksum and flags for "nodata" case.
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_mask_2():
 
     ds = gdal.Open("data/byte.vrt")
@@ -996,7 +984,10 @@ def test_mask_27():
 
 
 @pytest.mark.parametrize("dt", [gdal.GDT_Byte, gdal.GDT_Int64, gdal.GDT_UInt64])
-def test_mask_setting_nodata(dt):
+@pytest.mark.parametrize(
+    "GDAL_SIMUL_MEM_ALLOC_FAILURE_NODATA_MASK_BAND", [None, "YES", "ALWAYS"]
+)
+def test_mask_setting_nodata(dt, GDAL_SIMUL_MEM_ALLOC_FAILURE_NODATA_MASK_BAND):
     def set_nodata_value(ds, val):
         if dt == gdal.GDT_Byte:
             ds.GetRasterBand(1).SetNoDataValue(val)
@@ -1005,15 +996,41 @@ def test_mask_setting_nodata(dt):
         else:
             ds.GetRasterBand(1).SetNoDataValueAsUInt64(val)
 
-    ds = gdal.GetDriverByName("MEM").Create("", 1, 1, 1, dt)
-    assert ds.GetRasterBand(1).GetMaskBand().ReadRaster() == struct.pack("B", 255)
-    assert ds.GetRasterBand(1).GetMaskBand().ReadRaster() == struct.pack("B", 255)
-    set_nodata_value(ds, 0)
-    assert ds.GetRasterBand(1).GetMaskBand().ReadRaster() == struct.pack("B", 0)
-    assert ds.GetRasterBand(1).GetMaskBand().ReadRaster() == struct.pack("B", 0)
-    set_nodata_value(ds, 1)
-    assert ds.GetRasterBand(1).GetMaskBand().ReadRaster() == struct.pack("B", 255)
-    set_nodata_value(ds, 0)
-    assert ds.GetRasterBand(1).GetMaskBand().ReadRaster() == struct.pack("B", 0)
-    ds.GetRasterBand(1).DeleteNoDataValue()
-    assert ds.GetRasterBand(1).GetMaskBand().ReadRaster() == struct.pack("B", 255)
+    def test():
+        ds = gdal.GetDriverByName("MEM").Create("__debug__", 1, 1, 1, dt)
+        assert ds.GetRasterBand(1).GetMaskBand().ReadRaster() == struct.pack("B", 255)
+        assert ds.GetRasterBand(1).GetMaskBand().ReadRaster() == struct.pack("B", 255)
+        set_nodata_value(ds, 0)
+        got = ds.GetRasterBand(1).GetMaskBand().ReadRaster()
+        if (
+            GDAL_SIMUL_MEM_ALLOC_FAILURE_NODATA_MASK_BAND == "ALWAYS"
+            and dt != gdal.GDT_Byte
+        ):
+            assert got is None
+            assert gdal.GetLastErrorType() == gdal.CE_Failure
+        else:
+            if (
+                GDAL_SIMUL_MEM_ALLOC_FAILURE_NODATA_MASK_BAND == "YES"
+                and dt != gdal.GDT_Byte
+            ):
+                assert gdal.GetLastErrorType() == gdal.CE_Warning
+            assert got == struct.pack("B", 0)
+            assert ds.GetRasterBand(1).GetMaskBand().ReadRaster() == struct.pack("B", 0)
+            set_nodata_value(ds, 1)
+            assert ds.GetRasterBand(1).GetMaskBand().ReadRaster() == struct.pack(
+                "B", 255
+            )
+            set_nodata_value(ds, 0)
+            assert ds.GetRasterBand(1).GetMaskBand().ReadRaster() == struct.pack("B", 0)
+
+        ds.GetRasterBand(1).DeleteNoDataValue()
+        assert ds.GetRasterBand(1).GetMaskBand().ReadRaster() == struct.pack("B", 255)
+
+    if GDAL_SIMUL_MEM_ALLOC_FAILURE_NODATA_MASK_BAND:
+        with gdal.quiet_errors(), gdal.config_option(
+            "GDAL_SIMUL_MEM_ALLOC_FAILURE_NODATA_MASK_BAND",
+            GDAL_SIMUL_MEM_ALLOC_FAILURE_NODATA_MASK_BAND,
+        ):
+            test()
+    else:
+        test()
