@@ -11,23 +11,7 @@
 # Copyright (c) 2008, Frank Warmerdam <warmerdam@pobox.com>
 # Copyright (c) 2008-2013, Even Rouault <even dot rouault at spatialys.com>
 #
-# Permission is hereby granted, free of charge, to any person obtaining a
-# copy of this software and associated documentation files (the "Software"),
-# to deal in the Software without restriction, including without limitation
-# the rights to use, copy, modify, merge, publish, distribute, sublicense,
-# and/or sell copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included
-# in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-# DEALINGS IN THE SOFTWARE.
+# SPDX-License-Identifier: MIT
 ###############################################################################
 
 
@@ -70,6 +54,10 @@ def test_transformer_1():
 # Test GCP based transformer with polynomials.
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_transformer_2():
 
     ds = gdal.Open("data/gcps.vrt")
@@ -98,6 +86,10 @@ def test_transformer_2():
 # Test GCP based transformer with thin plate splines.
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_transformer_3():
 
     ds = gdal.Open("data/gcps.vrt")
@@ -126,6 +118,10 @@ def test_transformer_3():
 # Test geolocation based transformer.
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_transformer_4():
 
     ds = gdal.Open("data/sstgeo.vrt")
@@ -154,6 +150,10 @@ def test_transformer_4():
 # Test RPC based transformer.
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_transformer_5():
 
     ds = gdal.Open("data/rpc.vrt")
@@ -364,6 +364,10 @@ def test_transformer_5():
 # Test RPC convergence bug (bug # 5395)
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_transformer_6():
 
     ds = gdal.Open("data/rpc_5395.vrt")
@@ -402,6 +406,10 @@ def test_transformer_7():
 # Test handling of nodata in RPC DEM (#5680)
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_transformer_8():
 
     ds = gdal.Open("data/rpc.vrt")
@@ -448,6 +456,10 @@ def test_transformer_8():
 # Test RPC DEM line optimization
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_transformer_9():
 
     ds = gdal.Open("data/rpc.vrt")
@@ -504,6 +516,10 @@ def test_transformer_9():
 # Test RPC DEM transform from geoid height to ellipsoidal height
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 @pytest.mark.require_driver("GTX")
 def test_transformer_10():
 
@@ -637,6 +653,10 @@ def test_transformer_11():
 # Test degenerate cases of TPS transformer
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_transformer_12():
 
     ds = gdal.Open(
@@ -761,8 +781,8 @@ def test_transformer_13():
     (success, pnt) = tr.TransformPoint(0, 6600, 24)
     assert (
         success
-        and pnt[0] == pytest.approx(-108.00066000065341, abs=1e-7)
-        and pnt[1] == pytest.approx(39.157694013439489, abs=1e-7)
+        and pnt[0] == pytest.approx(-108.00069819119149, abs=1e-7)
+        and pnt[1] == pytest.approx(39.15771125604824, abs=1e-7)
     )
 
 
@@ -800,11 +820,14 @@ def test_transformer_14():
         tr = gdal.Transformer(
             ds, None, ["METHOD=RPC", "RPC_DEM=data/transformer_14_dem.tif"]
         )
-    (success, pnt) = tr.TransformPoint(0, 0, 0)
+    (success, pnt) = tr.TransformPoint(0, 45, 73)
+    # on debug it should say this two messages
+    # Oscillation detected...
+    # Converged!
     assert (
         success
-        and pnt[0] == pytest.approx(1.9391846640653961e-05, abs=1e-7)
-        and pnt[1] == pytest.approx(-0.0038824752244123275, abs=1e-7)
+        and pnt[0] == pytest.approx(0.000801360096912016, abs=1e-7)
+        and pnt[1] == pytest.approx(-4.1346931131054286e-05, abs=1e-7)
     )
 
     f = gdal.VSIFOpenL("/vsimem/transformer_14.csvt", "rb")
@@ -831,6 +854,10 @@ def test_transformer_14():
 # beyond
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_transformer_15():
 
     ds = gdal.GetDriverByName("MEM").Create("", 6600, 4400)
@@ -930,6 +957,10 @@ def test_transformer_15():
 # (we mostly test that the parameters are well recognized and serialized)
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_transformer_16():
 
     gdal.Translate(
@@ -963,6 +994,10 @@ def test_transformer_16():
 # Test RPC DEM with unexisting RPC DEM file
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_transformer_17():
 
     ds = gdal.Open("data/rpc.vrt")
@@ -1015,6 +1050,10 @@ def test_transformer_no_reverse_method():
 # Test precision of GCP based transformer with thin plate splines and lots of GCPs (2115).
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_transformer_tps_precision():
 
     ds = gdal.Open("data/gcps_2115.vrt")
@@ -1096,6 +1135,10 @@ def test_transformer_image_no_srs():
 # Test RPC_DEM_SRS by adding vertical component egm 96 geoid
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_transformer_dem_overrride_srs():
     ds = gdal.Open("data/rpc.vrt")
     ds_dem = gdal.GetDriverByName("GTiff").Create("/vsimem/dem.tif", 100, 100, 1)
@@ -1188,6 +1231,10 @@ def test_transformer_SuggestedWarpOutput_from_options():
 # Test GCP antimerdian unwrap (https://github.com/OSGeo/gdal/issues/8371)
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_transformer_gcp_antimeridian_unwrap():
 
     ds = gdal.Open("data/test_gcp_antimeridian_unwrap.vrt")
