@@ -10,23 +10,7 @@
 ###############################################################################
 # Copyright (c) 2010-2014, Even Rouault <even dot rouault at spatialys.com>
 #
-# Permission is hereby granted, free of charge, to any person obtaining a
-# copy of this software and associated documentation files (the "Software"),
-# to deal in the Software without restriction, including without limitation
-# the rights to use, copy, modify, merge, publish, distribute, sublicense,
-# and/or sell copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included
-# in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-# DEALINGS IN THE SOFTWARE.
+# SPDX-License-Identifier: MIT
 ###############################################################################
 
 import random
@@ -737,10 +721,16 @@ def test_vsizip_deflate64():
     assert f
     try:
         data = gdal.VSIFReadL(1, size, f)
+        assert gdal.VSIFEofL(f) == 0
+        assert gdal.VSIFErrorL(f) == 0
         assert len(data) == size
         assert len(gdal.VSIFReadL(1, 1, f)) == 0
+        assert gdal.VSIFEofL(f) == 1
+        assert gdal.VSIFErrorL(f) == 0
         assert gdal.VSIFSeekL(f, 0, 0) == 0
         data2 = gdal.VSIFReadL(1, size, f)
+        assert gdal.VSIFEofL(f) == 0
+        assert gdal.VSIFErrorL(f) == 0
         len_data2 = len(data2)
         assert len_data2 == size
         assert data2 == data
@@ -752,6 +742,11 @@ def test_vsizip_deflate64():
         ]:
             assert gdal.VSIFSeekL(f, pos, 0) == 0
             data2 = gdal.VSIFReadL(1, nread, f)
+            if pos + nread > size:
+                assert gdal.VSIFEofL(f) == 1
+            else:
+                assert gdal.VSIFEofL(f) == 0
+            assert gdal.VSIFErrorL(f) == 0, (pos, nread)
             len_data2 = len(data2)
             assert len_data2 == min(nread, size - pos), (pos, nread)
             assert data2 == data[pos : pos + len_data2], (pos, nread)
