@@ -10,26 +10,11 @@
 # Copyright (c) 2004, Frank Warmerdam <warmerdam@pobox.com>
 # Copyright (c) 2008-2011, Even Rouault <even dot rouault at spatialys.com>
 #
-# Permission is hereby granted, free of charge, to any person obtaining a
-# copy of this software and associated documentation files (the "Software"),
-# to deal in the Software without restriction, including without limitation
-# the rights to use, copy, modify, merge, publish, distribute, sublicense,
-# and/or sell copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included
-# in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-# DEALINGS IN THE SOFTWARE.
+# SPDX-License-Identifier: MIT
 ###############################################################################
 
 import os
+import shutil
 import struct
 
 import gdaltest
@@ -37,6 +22,8 @@ import pytest
 
 from osgeo import gdal
 
+pytestmark = pytest.mark.require_driver("HFA")
+
 ###############################################################################
 # Verify we can read the special histogram metadata from a provided image.
 
@@ -665,6 +652,10 @@ def test_hfa_vsimem():
 # the .img file.  (#2422)
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_hfa_proName():
 
     drv = gdal.GetDriverByName("HFA")
@@ -1237,21 +1228,17 @@ def test_hfa_write_rat():
 # Test STATISTICS creation option
 
 
-def test_hfa_createcopy_statistics():
+def test_hfa_createcopy_statistics(tmp_path):
 
-    tmpAuxXml = "../gcore/data/byte.tif.aux.xml"
-    try:
-        os.remove(tmpAuxXml)
-    except OSError:
-        pass
-    ds_src = gdal.Open("../gcore/data/byte.tif")
+    tmp_tif = str(tmp_path / "byte.tif")
+    shutil.copy("../gcore/data/byte.tif", tmp_tif)
+
+    ds_src = gdal.Open(tmp_tif)
     out_ds = gdal.GetDriverByName("HFA").CreateCopy(
         "/vsimem/byte.img", ds_src, options=["STATISTICS=YES"]
     )
     del out_ds
     ds_src = None
-    if os.path.exists(tmpAuxXml):
-        os.remove(tmpAuxXml)
 
     gdal.Unlink("/vsimem/byte.img.aux.xml")
 
