@@ -11,23 +11,7 @@
 # Copyright (c) 2005, Frank Warmerdam <warmerdam@pobox.com>
 # Copyright (c) 2009-2011, Even Rouault <even dot rouault at spatialys.com>
 #
-# Permission is hereby granted, free of charge, to any person obtaining a
-# copy of this software and associated documentation files (the "Software"),
-# to deal in the Software without restriction, including without limitation
-# the rights to use, copy, modify, merge, publish, distribute, sublicense,
-# and/or sell copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included
-# in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-# DEALINGS IN THE SOFTWARE.
+# SPDX-License-Identifier: MIT
 ###############################################################################
 
 import os
@@ -60,6 +44,7 @@ def test_pcidsk_1():
 # Test lossless copying (16, multiband) via Create().
 
 
+@pytest.mark.require_driver("PNG")
 def test_pcidsk_2():
 
     tst = gdaltest.GDALTest("PCIDSK", "png/rgba16.png", 2, 2042)
@@ -222,6 +207,7 @@ def test_pcidsk_5(tmp_path):
 # Test FILE interleaving.
 
 
+@pytest.mark.require_driver("PNG")
 def test_pcidsk_8():
 
     tst = gdaltest.GDALTest(
@@ -284,6 +270,7 @@ def test_pcidsk_10():
 # Test INTERLEAVING=TILED interleaving.
 
 
+@pytest.mark.require_driver("PNG")
 def test_pcidsk_11():
 
     tst = gdaltest.GDALTest(
@@ -297,6 +284,7 @@ def test_pcidsk_11():
     tst.testCreate()
 
 
+@pytest.mark.require_driver("PNG")
 def test_pcidsk_11_v1():
 
     tst = gdaltest.GDALTest(
@@ -310,6 +298,7 @@ def test_pcidsk_11_v1():
     tst.testCreate()
 
 
+@pytest.mark.require_driver("PNG")
 def test_pcidsk_11_v2():
 
     tst = gdaltest.GDALTest(
@@ -327,6 +316,7 @@ def test_pcidsk_11_v2():
 # Test INTERLEAVING=TILED interleaving and COMPRESSION=RLE
 
 
+@pytest.mark.require_driver("PNG")
 def test_pcidsk_12():
 
     tst = gdaltest.GDALTest(
@@ -340,6 +330,7 @@ def test_pcidsk_12():
     tst.testCreate()
 
 
+@pytest.mark.require_driver("PNG")
 def test_pcidsk_12_v1():
 
     tst = gdaltest.GDALTest(
@@ -358,6 +349,7 @@ def test_pcidsk_12_v1():
     tst.testCreate()
 
 
+@pytest.mark.require_driver("PNG")
 def test_pcidsk_12_v2():
 
     tst = gdaltest.GDALTest(
@@ -518,6 +510,8 @@ def test_pcidsk_external_ovr_rrd():
     with gdaltest.config_option("USE_RRD", "YES"):
         ds.BuildOverviews("NEAR", [2])
     ds = None
+    if gdal.GetLastErrorMsg() == "This build does not support creating .aux overviews":
+        pytest.skip(gdal.GetLastErrorMsg())
     assert gdal.VSIStatL("/vsimem/test.aux") is not None
     ds = gdal.Open("/vsimem/test.pix")
     assert ds.GetRasterBand(1).GetOverviewCount() == 1
