@@ -10,23 +10,7 @@
 ###############################################################################
 # Copyright (c) 2015, Even Rouault <even dot rouault at spatialys.com>
 #
-# Permission is hereby granted, free of charge, to any person obtaining a
-# copy of this software and associated documentation files (the "Software"),
-# to deal in the Software without restriction, including without limitation
-# the rights to use, copy, modify, merge, publish, distribute, sublicense,
-# and/or sell copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included
-# in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-# DEALINGS IN THE SOFTWARE.
+# SPDX-License-Identifier: MIT
 ###############################################################################
 
 import shutil
@@ -34,6 +18,7 @@ import struct
 
 import gdaltest
 import pytest
+import webserver
 
 from osgeo import gdal
 
@@ -879,6 +864,7 @@ def test_wmts_15():
             </TileMatrix>
         </TileMatrixSet>
     </Contents>
+    <ServiceMetadataURL xlink:href="/vsimem/do_not_use"/>
 </Capabilities>""",
     )
 
@@ -1870,6 +1856,18 @@ xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" version="1.0.0">
     gdal.Unlink(inputXml)
 
 
+###############################################################################
+# Test fix for https://github.com/OSGeo/gdal/issues/10348
+
+
+def test_wmts_clip_extent_with_union_of_tile_matrix_extent():
+
+    ds = gdal.Open("data/wmts/clip_WGS84BoundingBox_with_tilematrix.xml")
+    assert ds.GetGeoTransform() == pytest.approx(
+        (-46133.17, 0.5971642834779389, 0.0, 6301219.54, 0.0, -0.5971642834779389)
+    )
+
+
 ###############################################################################
 # Test when local wmts tiles are missing
 
@@ -1925,3 +1923,111 @@ def test_wmts_24():
     data = struct.unpack("h", structval)
     #   Expect a null value for the pixel data
     assert data[0] == 0
+
+
+###############################################################################
+# Test force opening a URL as WMTS
+
+
+def test_wmts_force_identifying_url():
+
+    drv = gdal.IdentifyDriverEx("http://example.com", allowed_drivers=["WMTS"])
+    assert drv.GetDescription() == "WMTS"
+
+
+# Launch a single webserver in a module-scoped fixture.
+@pytest.fixture(scope="module")
+def webserver_launch():
+
+    process, port = webserver.launch(handler=webserver.DispatcherHttpHandler)
+
+    yield process, port
+
+    webserver.server_stop(process, port)
+
+
+@pytest.fixture(scope="function")
+def webserver_port(webserver_launch):
+
+    webserver_process, webserver_port = webserver_launch
+
+    if webserver_port == 0:
+        pytest.skip()
+    yield webserver_port
+
+
+@pytest.mark.require_curl
+@gdaltest.enable_exceptions()
+def test_wmts_force_opening_url(tmp_vsimem, webserver_port):
+
+    handler = webserver.SequentialHandler()
+    handler.add(
+        "GET",
+        "/",
+        200,
+        {"Content-type": "application/xml"},
+        open("data/wmts/WMTSCapabilities.xml", "rb").read(),
+    )
+    with webserver.install_http_handler(handler):
+        gdal.OpenEx(f"http://localhost:{webserver_port}", allowed_drivers=["WMTS"])
+
+
+###############################################################################
+# Test force opening
+
+
+@gdaltest.enable_exceptions()
+def test_wmts_force_opening(tmp_vsimem):
+
+    filename = str(tmp_vsimem / "test.foo")
+
+    with open("data/wmts/WMTSCapabilities.xml", "rb") as fsrc:
+        with gdaltest.vsi_open(filename, "wb") as fdest:
+            fdest.write(fsrc.read(1))
+            fdest.write(b" " * (1000 * 1000))
+            fdest.write(fsrc.read())
+
+    with pytest.raises(Exception):
+        gdal.OpenEx(filename)
+
+    ds = gdal.OpenEx(filename, allowed_drivers=["WMTS"])
+    assert ds.GetDriver().GetDescription() == "WMTS"
+
+
+###############################################################################
+# Test force opening, but provided file is still not recognized (for good reasons)
+
+
+def test_wmts_force_opening_no_match():
+
+    drv = gdal.IdentifyDriverEx("data/byte.tif", allowed_drivers=["WMTS"])
+    assert drv is None
+
+
+###############################################################################
+# Test bug fix for https://github.com/OSGeo/gdal/issues/11387
+
+
+@pytest.mark.require_proj(9)
+@gdaltest.enable_exceptions()
+def test_wmts_read_esri_code_disguised_as_epsg_and_wrong_axis_order():
+
+    with gdaltest.error_handler():
+        with gdal.Open(
+            "data/wmts/WMTSCapabilities_THEMIS_NightIR_ControlledMosaics_100m_v2_oct2018.xml"
+        ) as ds:
+            assert gdal.GetLastErrorMsg().startswith(
+                "Auto-correcting wrongly swapped TileMatrix.TopLeftCorner coordinates"
+            )
+            assert ds.GetSpatialRef().GetAuthorityName(None) == "ESRI"
+            assert ds.GetSpatialRef().GetAuthorityCode(None) == "104905"
+            assert ds.GetGeoTransform() == pytest.approx(
+                (
+                    -180.0,
+                    0.0013717509172233527,
+                    0.0,
+                    65.00121128452162,
+                    0.0,
+                    -0.0013717509172233527,
+                )
+            )
