@@ -10,23 +10,7 @@
 ###############################################################################
 # Copyright (c) 2020, SAP SE
 #
-# Permission is hereby granted, free of charge, to any person obtaining a
-# copy of this software and associated documentation files (the "Software"),
-# to deal in the Software without restriction, including without limitation
-# the rights to use, copy, modify, merge, publish, distribute, sublicense,
-# and/or sell copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included
-# in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-# DEALINGS IN THE SOFTWARE.
+# SPDX-License-Identifier: MIT
 ###############################################################################
 from os import environ
 
@@ -424,7 +408,7 @@ def test_ogr_hana_18():
     feat_new = ogr.Feature(feature_def=layer.GetLayerDefn())
     feat_new.SetField("PRFEDEA", "9999")
     layer.CreateFeature(feat_new)
-    feat_new.Destroy()
+    feat_new = None
 
     layer.SetAttributeFilter("PRFEDEA = '9999'")
     feat = layer.GetNextFeature()
@@ -438,7 +422,7 @@ def test_ogr_hana_18():
     assert layer.SetFeature(feat) == 0, "SetFeature() method failed."
 
     fid = feat.GetFID()
-    feat.Destroy()
+    feat = None
 
     feat = layer.GetFeature(fid)
     assert feat is not None, "GetFeature(%d) failed." % fid
@@ -1170,6 +1154,93 @@ def test_ogr_hana_37():
     layer.CommitTransaction()
 
 
+###############################################################################
+# Test REAL_VECTOR type
+
+
+def test_ogr_hana_38():
+    conn = create_connection()
+    layer_name = get_test_name()
+    table_name = f'"{gdaltest.hana_schema_name}"."{layer_name}"'
+    execute_sql(
+        conn,
+        f"CREATE COLUMN TABLE {table_name} (ID INT PRIMARY KEY, EMB1 REAL_VECTOR(3), EMB2 REAL_VECTOR)",
+    )
+    execute_sql(
+        conn,
+        f"INSERT INTO {table_name} VALUES (1, TO_REAL_VECTOR('[0.1,0.2,0.3]'), TO_REAL_VECTOR('[0.1,0.2,0.3]'))",
+    )
+
+    def check_value(expected):
+        ds = open_datasource(0)
+        layer = ds.GetLayerByName(layer_name)
+        layer_defn = layer.GetLayerDefn()
+        assert layer.GetLayerDefn().GetFieldCount() == 2
+        field_emb1 = layer_defn.GetFieldDefn(layer_defn.GetFieldIndex("EMB1"))
+        assert field_emb1.GetType() == ogr.OFTBinary
+        assert field_emb1.GetWidth() == 16
+        field_emb2 = layer_defn.GetFieldDefn(layer_defn.GetFieldIndex("EMB2"))
+        assert field_emb2.GetType() == ogr.OFTBinary
+        assert field_emb2.GetWidth() == 65000
+        check_feature_count(layer, 1)
+        feat = layer.GetNextFeature()
+        assert feat.GetFieldAsBinary("EMB1") == expected
+        assert feat.GetFieldAsBinary("EMB2") == expected
+
+    # '[0.1,0.2,0.3]'
+    vec0 = b"\x03\x00\x00\x00\xCD\xCC\xCC\x3D\xCD\xCC\x4C\x3E\x9A\x99\x99\x3E"
+    # '[0.1,0.2,0.1]'
+    vec1 = b"\x03\x00\x00\x00\xCD\xCC\xCC\x3D\xCD\xCC\x4C\x3E\xCD\xCC\xCC\x3D"
+
+    check_value(vec0)
+
+    ds = open_datasource(1)
+    layer = ds.GetLayerByName(layer_name)
+    feat = layer.GetNextFeature()
+    feat.SetField("EMB1", vec1)
+    feat.SetField("EMB2", vec1)
+    layer.SetFeature(feat)
+
+    check_value(vec1)
+
+
+###############################################################################
+# Verify a working fallback in case the fast extent estimation fails
+
+
+def test_ogr_hana_39():
+    conn = create_connection()
+
+    # Create test table
+    layer_name = get_test_name()
+    table_name = f'"{gdaltest.hana_schema_name}"."{layer_name}"'
+    execute_sql(
+        conn,
+        f"CREATE COLUMN TABLE {table_name} (id INT, geom ST_Geometry(4326)) NO AUTO MERGE",
+    )
+
+    # Check extent. The table is empty so the extent should be (0, 0, 0, 0)
+    ds = open_datasource(0)
+    layer = ds.GetLayerByName(layer_name)
+    assert layer is not None, "did not get layer"
+    check_extent(layer, (0, 0, 0, 0), force=False)
+
+    # Insert points without merging the delta.
+    # The fallback should be triggered and return the correct extent.
+    execute_sql(
+        conn,
+        f"INSERT INTO {table_name} (id, geom) VALUES (0, ST_GeomFromText('POINT(0 10)', 4326))",
+    )
+    execute_sql(
+        conn,
+        f"INSERT INTO {table_name} (id, geom) VALUES (0, ST_GeomFromText('POINT(0 40)', 4326))",
+    )
+    check_extent(layer, (0, 0, 10, 40), force=False)
+
+    # Tear-down
+    execute_sql(conn, f"DROP TABLE {table_name}")
+
+
 ###############################################################################
 #  Create a table from data/poly.shp
 
@@ -1248,7 +1319,9 @@ def create_tpoly_table(ds, layer_name="TPOLY"):
 def get_connection_str():
     uri = gdal.GetConfigOption("OGR_HANA_CONNECTION_STRING", None)
     if uri is not None:
-        conn_str = uri + ";ENCRYPT=YES;SSL_VALIDATE_CERTIFICATE=false;CHAR_AS_UTF8=1"
+        if "ENCRYPT" not in uri:
+            uri += ";ENCRYPT=YES"
+        conn_str = uri + ";SSL_VALIDATE_CERTIFICATE=false;CHAR_AS_UTF8=1"
     else:
         pytest.skip("OGR_HANA_CONNECTION_STRING not set")
 
@@ -1311,8 +1384,8 @@ def open_datasource(update=0, open_opts=None):
         return gdal.OpenEx(conn_str, update, open_options=[open_opts])
 
 
-def check_extent(layer, expected, max_error=0.001):
-    actual = layer.GetExtent()
+def check_extent(layer, expected, force=True, max_error=0.001):
+    actual = layer.GetExtent(force=force)
     minx = abs(actual[0] - expected[0])
     maxx = abs(actual[1] - expected[1])
     miny = abs(actual[2] - expected[2])
