@@ -10,23 +10,7 @@
 # Copyright (c) 2003, Frank Warmerdam <warmerdam@pobox.com>
 # Copyright (c) 2008-2011, Even Rouault <even dot rouault at spatialys.com>
 #
-# Permission is hereby granted, free of charge, to any person obtaining a
-# copy of this software and associated documentation files (the "Software"),
-# to deal in the Software without restriction, including without limitation
-# the rights to use, copy, modify, merge, publish, distribute, sublicense,
-# and/or sell copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included
-# in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-# DEALINGS IN THE SOFTWARE.
+# SPDX-License-Identifier: MIT
 ###############################################################################
 
 
@@ -1464,6 +1448,18 @@ def test_ogr_mem_write_arrow():
     field_def = ogr.FieldDefn("field_stringlist", ogr.OFTStringList)
     src_lyr.CreateField(field_def)
 
+    field_def = ogr.FieldDefn("field_json", ogr.OFTString)
+    field_def.SetSubType(ogr.OFSTJSON)
+    src_lyr.CreateField(field_def)
+
+    field_def = ogr.FieldDefn("field_uuid", ogr.OFTString)
+    field_def.SetSubType(ogr.OFSTUUID)
+    src_lyr.CreateField(field_def)
+
+    field_def = ogr.FieldDefn("field_with_width", ogr.OFTString)
+    field_def.SetWidth(10)
+    src_lyr.CreateField(field_def)
+
     feat_def = src_lyr.GetLayerDefn()
     src_feature = ogr.Feature(feat_def)
     src_feature.SetField("field_bool", True)
@@ -1485,6 +1481,9 @@ def test_ogr_mem_write_arrow():
     src_feature.field_float32list = [1.5, -1.5]
     src_feature.field_reallist = [123.5, 567.0]
     src_feature.field_stringlist = ["abc", "def"]
+    src_feature["field_json"] = '{"foo":"bar"}'
+    src_feature["field_uuid"] = "INVALID_UUID"
+    src_feature["field_with_width"] = "foo"
     src_feature.SetGeometry(ogr.CreateGeometryFromWkt("POINT (1 2)"))
 
     src_lyr.CreateFeature(src_feature)
@@ -1506,6 +1505,15 @@ def test_ogr_mem_write_arrow():
         if schema.GetChild(i).GetName() != "wkb_geometry":
             dst_lyr.CreateFieldFromArrowSchema(schema.GetChild(i))
 
+    idx = dst_lyr.GetLayerDefn().GetFieldIndex("field_json")
+    assert dst_lyr.GetLayerDefn().GetFieldDefn(idx).GetSubType() == ogr.OFSTJSON
+
+    idx = dst_lyr.GetLayerDefn().GetFieldIndex("field_uuid")
+    assert dst_lyr.GetLayerDefn().GetFieldDefn(idx).GetSubType() == ogr.OFSTUUID
+
+    idx = dst_lyr.GetLayerDefn().GetFieldIndex("field_with_width")
+    assert dst_lyr.GetLayerDefn().GetFieldDefn(idx).GetWidth() == 10
+
     while True:
         array = stream.GetNextRecordBatch()
         if array is None:
@@ -2851,6 +2859,27 @@ def test_ogr_mem_write_pyarrow_invalid_dict_index(dict_values):
         lyr.WritePyArrow(table)
 
 
+###############################################################################
+
+
+def test_ogr_mem_arrow_json():
+    pytest.importorskip("pyarrow")
+
+    ds = ogr.GetDriverByName("Memory").CreateDataSource("")
+    lyr = ds.CreateLayer("foo")
+    field_def = ogr.FieldDefn("field_json", ogr.OFTString)
+    field_def.SetSubType(ogr.OFSTJSON)
+    lyr.CreateField(field_def)
+
+    stream = lyr.GetArrowStreamAsPyArrow()
+    field_schema = stream.schema["field_json"]
+    # Since pyarrow 18, the field type is extension<arrow.json>
+    if str(field_schema.type) != "extension<arrow.json>":
+        md = field_schema.metadata
+        assert b"ARROW:extension:name" in md
+        assert md[b"ARROW:extension:name"] == b"arrow.json"
+
+
 ###############################################################################
 # Test Layer.GetDataset()
 
