@@ -10,23 +10,7 @@
 ###############################################################################
 # Copyright (c) 2018, Even Rouault <even dot rouault at spatialys.com>
 #
-# Permission is hereby granted, free of charge, to any person obtaining a
-# copy of this software and associated documentation files (the "Software"),
-# to deal in the Software without restriction, including without limitation
-# the rights to use, copy, modify, merge, publish, distribute, sublicense,
-# and/or sell copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included
-# in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-# DEALINGS IN THE SOFTWARE.
+# SPDX-License-Identifier: MIT
 ###############################################################################
 
 
@@ -66,7 +50,15 @@ def test_ogr_oapif_errors():
     handler = webserver.SequentialHandler()
     handler.add("GET", "/oapif/collections", 404)
     with webserver.install_http_handler(handler):
-        with pytest.raises(Exception):
+        with pytest.raises(Exception, match="HTTP error code : 404"):
+            ogr.Open("OAPIF:http://localhost:%d/oapif" % gdaltest.webserver_port)
+
+    handler = webserver.SequentialHandler()
+    handler.add("GET", "/oapif/collections", 404, {}, "unavailable resource")
+    with webserver.install_http_handler(handler):
+        with pytest.raises(
+            Exception, match="HTTP error code : 404, unavailable resource"
+        ):
             ogr.Open("OAPIF:http://localhost:%d/oapif" % gdaltest.webserver_port)
 
     # No Content-Type
@@ -184,6 +176,9 @@ def test_ogr_oapif_empty_layer_and_user_query_parameters():
     assert lyr.GetName() == "foo"
 
     handler = webserver.SequentialHandler()
+    handler.add("GET", "/oapif?FOO=BAR", 200, {}, "{}")
+    handler.add("GET", "/oapif/api?FOO=BAR", 200, {}, "{}")
+    handler.add("GET", "/oapif/api/?FOO=BAR", 200, {}, "{}")
     handler.add(
         "GET",
         "/oapif/collections/foo/items?limit=20&FOO=BAR",
@@ -198,6 +193,15 @@ def test_ogr_oapif_empty_layer_and_user_query_parameters():
 ###############################################################################
 
 
+def _add_dummy_root_and_api_pages(handler, prefix=""):
+    handler.add("GET", prefix + "/oapif", 404, {}, "{}")
+    handler.add("GET", prefix + "/oapif/api", 404, {}, "{}")
+    handler.add("GET", prefix + "/oapif/api/", 404, {}, "{}")
+
+
+###############################################################################
+
+
 def test_ogr_oapif_open_by_collection_and_legacy_wfs3_prefix():
 
     handler = webserver.SequentialHandler()
@@ -218,6 +222,7 @@ def test_ogr_oapif_open_by_collection_and_legacy_wfs3_prefix():
     assert lyr.GetName() == "foo"
 
     handler = webserver.SequentialHandler()
+    _add_dummy_root_and_api_pages(handler)
     handler.add(
         "GET",
         "/oapif/collections/foo/items?limit=20",
@@ -237,19 +242,22 @@ def test_ogr_oapif_fc_links_next_geojson():
     handler = webserver.SequentialHandler()
     handler.add(
         "GET",
-        "/oapif/collections",
+        "/subdir/oapif/collections",
         200,
         {"Content-Type": "application/json"},
         '{ "collections" : [ { "name": "foo" }] }',
     )
     with webserver.install_http_handler(handler):
-        ds = ogr.Open("OAPIF:http://localhost:%d/oapif" % gdaltest.webserver_port)
+        ds = ogr.Open(
+            "OAPIF:http://localhost:%d/subdir/oapif" % gdaltest.webserver_port
+        )
     lyr = ds.GetLayer(0)
 
     handler = webserver.SequentialHandler()
+    _add_dummy_root_and_api_pages(handler, "/subdir")
     handler.add(
         "GET",
-        "/oapif/collections/foo/items?limit=20",
+        "/subdir/oapif/collections/foo/items?limit=20",
         200,
         {"Content-Type": "application/geo+json"},
         """{ "type": "FeatureCollection", "features": [
@@ -264,15 +272,16 @@ def test_ogr_oapif_fc_links_next_geojson():
     with webserver.install_http_handler(handler):
         assert lyr.GetLayerDefn().GetFieldCount() == 1
 
+    # Test relative links
     handler = webserver.SequentialHandler()
     handler.add(
         "GET",
-        "/oapif/collections/foo/items?limit=1000",
+        "/subdir/oapif/collections/foo/items?limit=1000",
         200,
         {"Content-Type": "application/geo+json"},
         """{ "type": "FeatureCollection",
                     "links" : [
-                        { "rel": "next", "type": "application/geo+json", "href": "http://localhost:%d/oapif/foo_next" }
+                        { "rel": "next", "type": "application/geo+json", "href": "/subdir/oapif/foo_next" }
                     ],
                     "features": [
                     {
@@ -281,22 +290,22 @@ def test_ogr_oapif_fc_links_next_geojson():
                             "foo": "bar"
                         }
                     }
-                ] }"""
-        % gdaltest.webserver_port,
+                ] }""",
     )
     with webserver.install_http_handler(handler):
         f = lyr.GetNextFeature()
-    if f["foo"] != "bar":
-        f.DumpReadable()
-        pytest.fail()
+    assert f["foo"] == "bar"
 
     handler = webserver.SequentialHandler()
     handler.add(
         "GET",
-        "/oapif/foo_next",
+        "/subdir/oapif/foo_next",
         200,
         {"Content-Type": "application/geo+json"},
         """{ "type": "FeatureCollection",
+                    "links" : [
+                        { "rel": "next", "type": "application/geo+json", "href": "./foo_next2" }
+                    ],
                     "features": [
                     {
                         "type": "Feature",
@@ -308,9 +317,73 @@ def test_ogr_oapif_fc_links_next_geojson():
     )
     with webserver.install_http_handler(handler):
         f = lyr.GetNextFeature()
-    if f["foo"] != "baz":
-        f.DumpReadable()
-        pytest.fail()
+    assert f["foo"] == "baz"
+
+    handler = webserver.SequentialHandler()
+    handler.add(
+        "GET",
+        "/subdir/oapif/foo_next2",
+        200,
+        {"Content-Type": "application/geo+json"},
+        """{ "type": "FeatureCollection",
+                    "links" : [
+                        { "rel": "next", "type": "application/geo+json", "href": "../oapif/foo_next3" }
+                    ],
+                    "features": [
+                    {
+                        "type": "Feature",
+                        "properties": {
+                            "foo": "baz2"
+                        }
+                    }
+                ] }""",
+    )
+    with webserver.install_http_handler(handler):
+        f = lyr.GetNextFeature()
+    assert f["foo"] == "baz2"
+
+    handler = webserver.SequentialHandler()
+    handler.add(
+        "GET",
+        "/subdir/oapif/foo_next3",
+        200,
+        {"Content-Type": "application/geo+json"},
+        """{ "type": "FeatureCollection",
+                    "links" : [
+                        { "rel": "next", "type": "application/geo+json", "href": "foo_next4" }
+                    ],
+                    "features": [
+                    {
+                        "type": "Feature",
+                        "properties": {
+                            "foo": "baz3"
+                        }
+                    }
+                ] }""",
+    )
+    with webserver.install_http_handler(handler):
+        f = lyr.GetNextFeature()
+    assert f["foo"] == "baz3"
+
+    handler = webserver.SequentialHandler()
+    handler.add(
+        "GET",
+        "/subdir/oapif/foo_next4",
+        200,
+        {"Content-Type": "application/geo+json"},
+        """{ "type": "FeatureCollection",
+                    "features": [
+                    {
+                        "type": "Feature",
+                        "properties": {
+                            "foo": "baz4"
+                        }
+                    }
+                ] }""",
+    )
+    with webserver.install_http_handler(handler):
+        f = lyr.GetNextFeature()
+    assert f["foo"] == "baz4"
 
 
 ###############################################################################
@@ -331,6 +404,7 @@ def test_ogr_oapif_id_is_integer():
     lyr = ds.GetLayer(0)
 
     handler = webserver.SequentialHandler()
+    _add_dummy_root_and_api_pages(handler)
     handler.add(
         "GET",
         "/oapif/collections/foo/items?limit=20",
@@ -521,6 +595,7 @@ def test_ogr_oapif_spatial_filter():
     assert lyr.GetExtent() == (-10.0, 15.0, 40.0, 50.0)
 
     handler = webserver.SequentialHandler()
+    _add_dummy_root_and_api_pages(handler)
     handler.add(
         "GET",
         "/oapif/collections/foo/items?limit=20",
@@ -770,6 +845,7 @@ def test_ogr_oapif_limit_from_numberMatched():
     lyr = ds.GetLayer(0)
 
     handler = webserver.SequentialHandler()
+    _add_dummy_root_and_api_pages(handler)
     handler.add(
         "GET",
         "/oapif/collections/foo/items?limit=20",
@@ -1112,6 +1188,7 @@ def test_ogr_oapif_schema_from_xml_schema():
 </xs:complexType>
 </xs:schema>""",
     )
+    _add_dummy_root_and_api_pages(handler)
     handler.add(
         "GET",
         "/oapif/collections/foo/items?limit=20",
@@ -1161,6 +1238,7 @@ def test_ogr_oapif_schema_from_json_schema():
         {"Content-Type": "application/schema+json"},
         open("data/oapif/oapif_json_schema_eo.jsonschema", "rt").read(),
     )
+    _add_dummy_root_and_api_pages(handler)
     handler.add(
         "GET",
         "/oapif/collections/foo/items?limit=20",
@@ -1219,6 +1297,7 @@ def test_ogr_oapif_stac_catalog():
     lyr = ds.GetLayer(0)
 
     handler = webserver.SequentialHandler()
+    _add_dummy_root_and_api_pages(handler)
     handler.add(
         "GET",
         "/oapif/collections/foo/items?limit=20",
@@ -1294,6 +1373,7 @@ def test_ogr_oapif_storage_crs_easting_northing():
     )
 
     handler = webserver.SequentialHandler()
+    _add_dummy_root_and_api_pages(handler)
     handler.add(
         "GET",
         "/oapif/collections/foo/items?limit=20",
@@ -1392,6 +1472,7 @@ def test_ogr_oapif_storage_crs_latitude_longitude():
     assert (minx, miny, maxx, maxy) == pytest.approx((-10, 40, 15, 50), abs=1e-3)
 
     handler = webserver.SequentialHandler()
+    _add_dummy_root_and_api_pages(handler)
     handler.add(
         "GET",
         "/oapif/collections/foo/items?limit=20",
@@ -1500,6 +1581,7 @@ def test_ogr_oapif_storage_crs_latitude_longitude_non_compliant_server():
     assert supported_srs_list is None
 
     handler = webserver.SequentialHandler()
+    _add_dummy_root_and_api_pages(handler)
     handler.add(
         "GET",
         "/oapif/collections/foo/items?limit=20",
@@ -1599,6 +1681,7 @@ def test_ogr_oapif_crs_and_preferred_crs_open_options():
 
     def get_items_handler():
         handler = webserver.SequentialHandler()
+        _add_dummy_root_and_api_pages(handler)
         handler.add(
             "GET",
             "/oapif/collections/foo/items?limit=20",
@@ -1968,11 +2051,14 @@ def test_ogr_oapif_initial_request_page_size():
     )
     with webserver.install_http_handler(handler):
         ds = gdal.OpenEx(
-            "OAPIF:http://localhost:%d/oapif" % gdaltest.webserver_port, gdal.OF_VECTOR
+            "http://localhost:%d/oapif" % gdaltest.webserver_port,
+            gdal.OF_VECTOR,
+            allowed_drivers=["OAPIF"],
         )
     lyr = ds.GetLayer(0)
 
     handler = webserver.SequentialHandler()
+    _add_dummy_root_and_api_pages(handler)
     handler.add(
         "GET",
         "/oapif/collections/foo/items?limit=20",
@@ -2009,6 +2095,7 @@ def test_ogr_oapif_initial_request_page_size():
     lyr = ds.GetLayer(0)
 
     handler = webserver.SequentialHandler()
+    _add_dummy_root_and_api_pages(handler)
     handler.add(
         "GET",
         "/oapif/collections/foo/items?limit=20",
@@ -2044,6 +2131,7 @@ def test_ogr_oapif_initial_request_page_size():
     lyr = ds.GetLayer(0)
 
     handler = webserver.SequentialHandler()
+    _add_dummy_root_and_api_pages(handler)
     handler.add(
         "GET",
         "/oapif/collections/foo/items?limit=1000",
@@ -2078,6 +2166,7 @@ def test_ogr_oapif_initial_request_page_size():
     lyr = ds.GetLayer(0)
 
     handler = webserver.SequentialHandler()
+    _add_dummy_root_and_api_pages(handler)
     handler.add(
         "GET",
         "/oapif/collections/foo/items?limit=30",
@@ -2094,3 +2183,82 @@ def test_ogr_oapif_initial_request_page_size():
     )
     with webserver.install_http_handler(handler):
         assert lyr.GetLayerDefn().GetFieldCount() == 1
+
+
+def test_ogr_oapif_datetime_open_option():
+    """Test DATETIME open option"""
+
+    handler = webserver.SequentialHandler()
+    handler.add(
+        "GET",
+        "/oapif/collections",
+        200,
+        {"Content-Type": "application/json"},
+        '{ "collections" : [ { "name": "foo" }] }',
+    )
+    with webserver.install_http_handler(handler):
+        ds = gdal.OpenEx(
+            "http://localhost:%d/oapif" % gdaltest.webserver_port,
+            gdal.OF_VECTOR,
+            open_options=["DATETIME=2011-01-03T12:31:00Z"],
+            allowed_drivers=["OAPIF"],
+        )
+    lyr = ds.GetLayer(0)
+
+    handler = webserver.SequentialHandler()
+    _add_dummy_root_and_api_pages(handler)
+    handler.add(
+        "GET",
+        "/oapif/collections/foo/items?limit=20",
+        200,
+        {"Content-Type": "application/geo+json"},
+        """{ "type": "FeatureCollection", "features": [
+                    {
+                        "type": "Feature",
+                        "properties": {
+                            "foo": "bar"
+                        }
+                    }
+                ] }""",
+    )
+    handler.add(
+        "GET",
+        "/oapif/collections/foo/items?limit=1000&datetime=2011-01-03T12:31:00Z",
+        200,
+        {"Content-Type": "application/geo+json"},
+        """{ "type": "FeatureCollection", "features": [
+                    {
+                        "type": "Feature",
+                        "properties": {
+                            "foo": "bar"
+                        }
+                    },
+                    {
+                        "type": "Feature",
+                        "properties": {
+                            "bar": "baz"
+                        }
+                    }
+                ] }""",
+    )
+    with webserver.install_http_handler(handler):
+        assert lyr.GetFeatureCount() == 2
+
+    handler = webserver.SequentialHandler()
+    handler.add(
+        "GET",
+        "/oapif/collections/foo/items?limit=1000&datetime=2011-01-03T12:31:00Z",
+        200,
+        {"Content-Type": "application/geo+json"},
+        """{ "type": "FeatureCollection", "features": [
+                    {
+                        "type": "Feature",
+                        "properties": {
+                            "foo": "bar"
+                        }
+                    }
+                ] }""",
+    )
+    with webserver.install_http_handler(handler):
+        f = lyr.GetNextFeature()
+        assert f["foo"] == "bar"
