@@ -27,6 +27,7 @@
 # Boston, MA 02111-1307, USA.
 ###############################################################################
 
+import math
 import os
 import shutil
 
@@ -36,14 +37,6 @@ import pytest
 
 from osgeo import gdal, ogr
 
-
-###############################################################################
-@pytest.fixture(autouse=True, scope="module")
-def module_disable_exceptions():
-    with gdaltest.disable_exceptions():
-        yield
-
-
 ###############################################################################
 # Test ExecuteSQL()
 
@@ -109,7 +102,9 @@ def test_ogr_sql_execute_sql(tmp_path, use_gdal):
     with pytest.raises(Exception):
         lyr.GetName()
 
-    assert get_lyr().GetFeatureCount() == 10
+    # This leaks memory
+    if not gdaltest.is_travis_branch("sanitize"):
+        assert get_lyr().GetFeatureCount() == 10
 
     # Check that we can actually remove the files (i.e. references on dataset have been dropped)
     os.unlink(tmp_path / "test_ogr_sql_execute_sql.shp")
@@ -262,12 +257,14 @@ def test_ogr_sql_4(data_ds):
 def test_ogr_sql_5(data_ds):
 
     with data_ds.ExecuteSQL(
-        "select max(eas_id), min(eas_id), avg(eas_id), sum(eas_id), count(eas_id) from idlink"
+        "select max(eas_id), min(eas_id), avg(eas_id), STDDEV_POP(eas_id), STDDEV_SAMP(eas_id), sum(eas_id), count(eas_id) from idlink"
     ) as sql_lyr:
         feat = sql_lyr.GetNextFeature()
         assert feat["max_eas_id"] == 179
         assert feat["min_eas_id"] == 158
         assert feat["avg_eas_id"] == pytest.approx(168.142857142857, abs=1e-12)
+        assert feat["STDDEV_POP_eas_id"] == pytest.approx(5.9384599116647205, rel=1e-15)
+        assert feat["STDDEV_SAMP_eas_id"] == pytest.approx(6.414269805898183, rel=1e-15)
         assert feat["count_eas_id"] == 7
         assert feat["sum_eas_id"] == 1177
 
@@ -424,6 +421,7 @@ def test_ogr_sql_13(data_ds):
 # Verify selection of, and on ogr_style and ogr_geom_wkt.
 
 
+@pytest.mark.require_driver("MapInfo File")
 def test_ogr_sql_14():
 
     expect = [
@@ -457,6 +455,7 @@ def test_ogr_sql_15(data_ds):
 ###############################################################################
 
 
+@pytest.mark.require_driver("MapInfo File")
 def test_ogr_sql_16():
 
     expect = [2]
@@ -470,6 +469,7 @@ def test_ogr_sql_16():
 ###############################################################################
 # Test the RFC 21 CAST operator.
 #
+@pytest.mark.require_driver("MapInfo File")
 def test_ogr_sql_17():
 
     expect = ["1", "2"]
@@ -497,16 +497,6 @@ def test_ogr_sql_17():
         ogrtest.check_features_against_list(sql_lyr, "fid", expect)
 
 
-###############################################################################
-# Test empty request string
-
-
-def test_ogr_sql_19(data_ds):
-
-    with gdal.quiet_errors():
-        assert data_ds.ExecuteSQL("") is None
-
-
 ###############################################################################
 # Test query "SELECT * from my_layer" on layer without any field (#2788)
 
@@ -686,7 +676,8 @@ def test_ogr_sql_27():
 # code from the grammar
 
 
-def test_ogr_sql_28():
+@pytest.fixture(scope="module")
+def ds_for_invalid_statements():
 
     ds = ogr.GetDriverByName("Memory").CreateDataSource("my_ds")
     lyr = ds.CreateLayer("my_layer")
@@ -710,10 +701,13 @@ def test_ogr_sql_28():
     field_defn = ogr.FieldDefn("strfield2", ogr.OFTString)
     lyr.CreateField(field_defn)
 
-    with pytest.raises(Exception):
-        sql_lyr = ds.ExecuteSQL(None)
+    yield ds
 
-    queries = [
+
+@pytest.mark.parametrize(
+    "sql",
+    [
+        None,
         "",
         "1",
         "*",
@@ -728,6 +722,8 @@ def test_ogr_sql_28():
         "SELECT 1 - FROM my_layer",
         "SELECT 1 * FROM my_layer",
         "SELECT 1 % FROM my_layer",
+        "SELECT x.",
+        "SELECT x AS",
         "SELECT *",
         "SELECT * FROM",
         "SELECT * FROM foo",
@@ -784,8 +780,10 @@ def test_ogr_sql_28():
         "SELECT COUNT(*) FROM",
         "SELECT COUNT(*) AS foo FROM",
         "SELECT COUNT(* FROM my_layer",
+        "SELECT COUNT(i_dont_exist) FROM my_layer",
         "SELECT COUNT(FOO intfield) FROM my_layer",
         "SELECT COUNT(DISTINCT intfield FROM my_layer",
+        "SELECT COUNT(DISTINCT i_dont_exist) FROM my_layer",
         "SELECT COUNT(DISTINCT *) FROM my_layer",
         "SELECT FOO(DISTINCT intfield) FROM my_layer",
         "SELECT FOO(DISTINCT intfield) as foo FROM my_layer",
@@ -800,10 +798,15 @@ def test_ogr_sql_28():
         "SELECT MAX(foo) FROM my_layer",
         "SELECT SUM(foo) FROM my_layer",
         "SELECT AVG(foo) FROM my_layer",
+        "SELECT STDDEV_POP(foo) FROM my_layer",
+        "SELECT STDDEV_SAMP(foo) FROM my_layer",
         "SELECT SUM(strfield) FROM my_layer",
         "SELECT AVG(strfield) FROM my_layer",
         "SELECT AVG(intfield, intfield) FROM my_layer",
+        "SELECT STDDEV_POP(strfield) FROM my_layer",
+        "SELECT STDDEV_SAMP(strfield) FROM my_layer",
         "SELECT * FROM my_layer WHERE AVG(intfield) = 1",
+        "SELECT * FROM my_layer WHERE STDDEV_POP(intfield) = 1",
         "SELECT * FROM 'foo' foo",
         "SELECT * FROM my_layer WHERE strfield =",
         "SELECT * FROM my_layer WHERE strfield = foo",
@@ -839,19 +842,12 @@ def test_ogr_sql_28():
         "SELECT * FROM my_layer UNION ALL SELECT",
         "SELECT * FROM my_layer UNION ALL SELECT *",
         "SELECT * FROM my_layer UNION ALL SELECT * FROM",
-    ]
+    ],
+)
+def test_ogr_sql_invalid_statements(ds_for_invalid_statements, sql):
 
-    for query in queries:
-        gdal.ErrorReset()
-        # print query
-        with gdal.quiet_errors():
-            sql_lyr = ds.ExecuteSQL(query)
-        if sql_lyr is not None:
-            ds.ReleaseResultSet(sql_lyr)
-            pytest.fail('expected None result on "%s"' % query)
-        assert gdal.GetLastErrorType() != 0, 'expected error on "%s"' % query
-
-    ds = None
+    with pytest.raises(Exception):
+        ds_for_invalid_statements.ExecuteSQL(None)
 
 
 ###############################################################################
@@ -1017,11 +1013,8 @@ def test_ogr_sql_34(data_ds):
 
         assert val == 1
 
-    with gdal.quiet_errors():
-        assert (
-            data_ds.ExecuteSQL("select count(*) from poly where eas_id in ('a165')")
-            is None
-        )
+    with pytest.raises(Exception):
+        data_ds.ExecuteSQL("select count(*) from poly where eas_id in ('a165')")
 
 
 ###############################################################################
@@ -1084,7 +1077,7 @@ def test_ogr_sql_36():
 # Test select count([distinct] column) with null values (#4354)
 
 
-def test_ogr_sql_37():
+def test_ogr_sql_count_and_null():
 
     ds = ogr.GetDriverByName("Memory").CreateDataSource("ogr_sql_37")
     lyr = ds.CreateLayer("layer")
@@ -1122,10 +1115,11 @@ def test_ogr_sql_37():
             assert feat.GetFieldAsInteger(2) == 4, fieldname
 
     with ds.ExecuteSQL(
-        "select avg(intfield) from layer where intfield is null"
+        "select avg(intfield), STDDEV_POP(intfield) from layer where intfield is null"
     ) as sql_lyr:
         feat = sql_lyr.GetNextFeature()
         assert feat.IsFieldSetAndNotNull(0) == 0
+        assert feat.IsFieldSetAndNotNull(1) == 0
 
     # Fix crash when first values is null (#4509)
     with ds.ExecuteSQL("select distinct strfield_first_null from layer") as sql_lyr:
@@ -1217,11 +1211,11 @@ def test_ogr_sql_42(data_ds):
 
 def test_ogr_sql_43(data_ds):
 
-    sql = "SELECT '\"' as a, '\\'' as b, '''' as c FROM poly"
+    sql = "SELECT '\"' as a, '\\' as b, '''' as c FROM poly"
     with data_ds.ExecuteSQL(sql) as sql_lyr:
         feat = sql_lyr.GetNextFeature()
         assert feat["a"] == '"'
-        assert feat["b"] == "'"
+        assert feat["b"] == "\\"
         assert feat["c"] == "'"
 
 
@@ -1229,19 +1223,23 @@ def test_ogr_sql_43(data_ds):
 # Test hstore_get_value()
 
 
-def test_ogr_sql_44(data_ds):
-
-    # Invalid parameters
-    for sql in [
+@pytest.mark.parametrize(
+    "sql",
+    [
         "SELECT hstore_get_value('a') FROM poly",
         "SELECT hstore_get_value(1, 1) FROM poly",
-    ]:
-        with gdal.quiet_errors():
-            sql_lyr = data_ds.ExecuteSQL(sql)
-        assert sql_lyr is None, sql
+    ],
+)
+def test_ogr_sql_hstore_get_value_invalid_parameters(data_ds, sql):
 
-    # Invalid hstore syntax or empty result
-    for sql in [
+    # Invalid parameters
+    with pytest.raises(Exception):
+        data_ds.ExecuteSQL(sql)
+
+
+@pytest.mark.parametrize(
+    "sql",
+    [
         "SELECT hstore_get_value('a', null) FROM poly",
         "SELECT hstore_get_value(null, 'a') FROM poly",
         "SELECT hstore_get_value('a', 'a') FROM poly",
@@ -1256,13 +1254,19 @@ def test_ogr_sql_44(data_ds):
         "SELECT hstore_get_value('\"a\" => ', 'a') FROM poly",
         "SELECT hstore_get_value('\"a\" => \"', 'a') FROM poly",
         "SELECT hstore_get_value('\"a\" => \"\" z', 'a') FROM poly",
-    ]:
-        with data_ds.ExecuteSQL(sql) as sql_lyr:
-            f = sql_lyr.GetNextFeature()
-            assert not f.IsFieldSetAndNotNull(0), sql
+    ],
+)
+def test_ogr_sql_hstore_get_value_invalid_hstore_syntax_or_empty_result(data_ds, sql):
 
-    # Valid hstore syntax
-    for (sql, expected) in [
+    # Invalid hstore syntax or empty result
+    with data_ds.ExecuteSQL(sql) as sql_lyr:
+        f = sql_lyr.GetNextFeature()
+        assert not f.IsFieldSetAndNotNull(0), sql
+
+
+@pytest.mark.parametrize(
+    "sql,expected",
+    [
         ("SELECT hstore_get_value('a=>b', 'a') FROM poly", "b"),
         ("SELECT hstore_get_value(' a => b ', 'a') FROM poly", "b"),
         ("SELECT hstore_get_value('\"a\"=>b', 'a') FROM poly", "b"),
@@ -1272,16 +1276,21 @@ def test_ogr_sql_44(data_ds):
         ("SELECT hstore_get_value('\"a\"=>\"b\"', 'a') FROM poly", "b"),
         ("SELECT hstore_get_value(' \"a\" => \"b\" ', 'a') FROM poly", "b"),
         ('SELECT hstore_get_value(\' "a\\"b" => "b" \', \'a"b\') FROM poly', "b"),
-    ]:
-        with data_ds.ExecuteSQL(sql) as sql_lyr:
-            f = sql_lyr.GetNextFeature()
-            assert f.GetField(0) == expected, sql
+    ],
+)
+def test_ogr_sql_hstore_get_value_valid(data_ds, sql, expected):
+
+    # Valid hstore syntax
+    with data_ds.ExecuteSQL(sql) as sql_lyr:
+        f = sql_lyr.GetNextFeature()
+        assert f.GetField(0) == expected, sql
 
 
 ###############################################################################
 # Test 64 bit GetFeatureCount()
 
 
+@pytest.mark.require_driver("OGR_VRT")
 def test_ogr_sql_45():
 
     ds = ogr.Open(
@@ -1315,7 +1324,8 @@ def test_ogr_sql_45():
 # Test strict SQL quoting
 
 
-def test_ogr_sql_46():
+@pytest.fixture(scope="module")
+def ogr_sql_strit_quoting_ds():
 
     ds = ogr.GetDriverByName("Memory").CreateDataSource("test")
     lyr = ds.CreateLayer("test")
@@ -1329,9 +1339,13 @@ def test_ogr_sql_46():
     feat.SetField(0, 3)
     feat.SetField(1, "from")
     lyr.CreateFeature(feat)
-    feat = None
 
-    with ds.ExecuteSQL(
+    return ds
+
+
+def test_ogr_sql_strict_quoting_non_aggregate(ogr_sql_strit_quoting_ds):
+
+    with ogr_sql_strit_quoting_ds.ExecuteSQL(
         'select id, \'id\', "id" as id2, id as "id3", "from" from test where "from" = \'from\''
     ) as sql_lyr:
         feat = sql_lyr.GetNextFeature()
@@ -1344,7 +1358,10 @@ def test_ogr_sql_46():
         feat = sql_lyr.GetNextFeature()
         assert feat is None
 
-    with ds.ExecuteSQL(
+
+def test_ogr_sql_strict_quoting_aggregate(ogr_sql_strit_quoting_ds):
+
+    with ogr_sql_strit_quoting_ds.ExecuteSQL(
         'select max("id"), max(id), count("id"), count(id) from "test"'
     ) as sql_lyr:
         feat = sql_lyr.GetNextFeature()
@@ -1353,16 +1370,20 @@ def test_ogr_sql_46():
         assert feat.GetField(2) == 2
         assert feat.GetField(3) == 2
 
-    # Not accepted
-    for sql in [
+
+@pytest.mark.parametrize(
+    "sql",
+    [
         "select * from 'test'",
         "select distinct 'id' from 'test'",
         "select max('id') from 'test'",
         "select id as 'id2' from 'test'",
-    ]:
-        with gdal.quiet_errors():
-            sql_lyr = ds.ExecuteSQL("select * from 'test'")
-        assert sql_lyr is None, sql
+    ],
+)
+def test_ogr_sql_strict_quoting_errors(ogr_sql_strit_quoting_ds, sql):
+
+    with pytest.raises(Exception):
+        ogr_sql_strit_quoting_ds.ExecuteSQL(sql)
 
 
 ###############################################################################
@@ -1596,9 +1617,8 @@ def test_ogr_sql_min_max_string_field():
     ],
 )
 def test_ogr_sql_select_except_errors(data_ds, body):
-    with gdal.quiet_errors():
-        lyr = data_ds.ExecuteSQL(f"SELECT * EXCEPT {body} FROM poly")
-    assert lyr is None
+    with pytest.raises(Exception):
+        data_ds.ExecuteSQL(f"SELECT * EXCEPT {body} FROM poly")
 
 
 def test_ogr_sql_select_except_attrs(data_ds):
@@ -1863,7 +1883,6 @@ def test_ogr_sql_ilike_utf8():
 # Test error on setting a spatial filter during ExecuteSQL
 
 
-@gdaltest.enable_exceptions()
 def test_ogr_sql_test_execute_sql_error_on_spatial_filter_mem_layer():
 
     ds = ogr.GetDriverByName("Memory").CreateDataSource("")
@@ -1873,3 +1892,266 @@ def test_ogr_sql_test_execute_sql_error_on_spatial_filter_mem_layer():
         Exception, match="Cannot set spatial filter: no geometry field present in layer"
     ):
         ds.ExecuteSQL("SELECT 1 FROM test", spatialFilter=geom)
+
+
+###############################################################################
+# Test NOT/IN on a comparison where a NULL value was involved
+
+
+@pytest.fixture(scope="module")
+def ds_for_test_ogr_sql_on_null():
+
+    ds = ogr.GetDriverByName("Memory").CreateDataSource("test_ogr_sql_on_null")
+    lyr = ds.CreateLayer("layer")
+    lyr.CreateField(ogr.FieldDefn("intfield", ogr.OFTInteger))
+    lyr.CreateField(ogr.FieldDefn("realfield", ogr.OFTReal))
+    lyr.CreateField(ogr.FieldDefn("datetimefield", ogr.OFTDateTime))
+    lyr.CreateField(ogr.FieldDefn("strfield", ogr.OFTString))
+    feat = ogr.Feature(lyr.GetLayerDefn())
+    lyr.CreateFeature(feat)
+    feat = ogr.Feature(lyr.GetLayerDefn())
+    feat["intfield"] = 1
+    feat["realfield"] = 1
+    feat["datetimefield"] = "2024-01-01T00:00:00"
+    feat["strfield"] = "foo"
+    lyr.CreateFeature(feat)
+
+    yield ds
+
+
+def get_available_dialects():
+    return [None, "SQLite"] if ogr.GetDriverByName("SQLite") else [None]
+
+
+@pytest.mark.parametrize(
+    "where,feature_count",
+    [  # intfield
+        ("1 + intfield >= 0", 1),
+        ("intfield = 0", 0),
+        ("intfield = 1", 1),
+        ("NOT intfield = 0", 1),
+        ("NOT intfield = 1", 0),
+        ("intfield IS NULL", 1),
+        ("intfield IS NOT NULL", 1),
+        ("intfield IN (NULL)", 0),
+        ("NULL IN (NULL)", 0),
+        ("NULL NOT IN (NULL)", 0),
+        ("intfield NOT IN (NULL)", 0),
+        ("intfield IN (1, NULL)", 1),
+        ("intfield IN (0, NULL)", 0),
+        ("intfield IN (NULL, 1)", 1),
+        ("intfield IN (NULL, 0)", 0),
+        ("intfield NOT IN (1, NULL)", 0),
+        ("intfield NOT IN (0, NULL)", 0),
+        ("intfield NOT IN (NULL, 1)", 0),
+        ("intfield NOT IN (NULL, 0)", 0),
+        ("(NOT intfield = 0) OR intfield IS NULL", 2),
+        ("NOT (intfield = 0 OR intfield = 0)", 1),
+        ("(NOT intfield = 0) AND NOT (intfield = 0)", 1),
+        ("NOT (intfield = 0 OR intfield IS NULL)", 1),
+        ("(NOT intfield = 0) AND NOT (intfield IS NULL)", 1),
+        ("NOT (intfield = 0 OR intfield IS NOT NULL)", 0),
+        ("(NOT intfield = 0) AND NOT (intfield IS NOT NULL)", 0),
+        ("intfield <> 0 AND intfield <> 2", 1),
+        ("intfield IS NOT NULL AND intfield NOT IN (2)", 1),
+        ("NOT(intfield NOT IN (1) AND NULL NOT IN (1))", 1),
+        ("NOT(intfield IS NOT NULL AND intfield NOT IN (2))", 1),
+        ("NOT(NOT(intfield IS NOT NULL AND intfield NOT IN (2)))", 1),
+        ("NOT (intfield = 0 AND intfield = 0)", 1),
+        ("(intfield NOT IN (1) AND NULL NOT IN (1)) IS NULL", 1),
+        # realfield
+        ("1 + realfield >= 0", 1),
+        ("realfield = 0", 0),
+        ("realfield = 1", 1),
+        ("NOT realfield = 0", 1),
+        ("NOT realfield = 1", 0),
+        ("realfield IS NULL", 1),
+        ("realfield IS NOT NULL", 1),
+        ("realfield IN (NULL)", 0),
+        ("realfield NOT IN (NULL)", 0),
+        ("realfield IN (1, NULL)", 1),
+        ("realfield IN (0, NULL)", 0),
+        ("realfield NOT IN (1, NULL)", 0),
+        ("realfield NOT IN (0, NULL)", 0),
+        ("(NOT realfield = 0) OR realfield IS NULL", 2),
+        ("NOT (realfield = 0 OR realfield = 0)", 1),
+        ("NOT (realfield = 0 OR realfield IS NULL)", 1),
+        ("NOT (realfield = 0 OR realfield IS NOT NULL)", 0),
+        # strfield
+        ("strfield = ''", 0),
+        ("strfield = 'foo'", 1),
+        ("NOT strfield = ''", 1),
+        ("NOT strfield = 'foo'", 0),
+        ("strfield IS NULL", 1),
+        ("strfield IS NOT NULL", 1),
+        ("strfield IN ('foo', NULL)", 1),
+        ("strfield NOT IN ('foo', NULL)", 0),
+        ("strfield IN ('', NULL)", 0),
+        ("strfield NOT IN ('', NULL)", 0),
+        # datetimefield
+        ("datetimefield = '1970-01-01T00:00:00'", 0),
+        ("datetimefield = '2024-01-01T00:00:00'", 1),
+        ("NOT datetimefield = '1970-01-01T00:00:00'", 1),
+        ("NOT datetimefield = '2024-01-01T00:00:00'", 0),
+        ("datetimefield IS NULL", 1),
+        ("datetimefield IS NOT NULL", 1),
+        ("datetimefield IN ('2024-01-01T00:00:00', NULL)", 1),
+        ("datetimefield NOT IN ('2024-01-01T00:00:00', NULL)", 0),
+        ("datetimefield IN ('1970-01-01T00:00:00', NULL)", 0),
+        ("datetimefield NOT IN ('1970-01-01T00:00:00', NULL)", 0),
+        ("datetimefield IN ('invalid', NULL)", None),
+    ],
+)
+@pytest.mark.parametrize("dialect", get_available_dialects())
+def test_ogr_sql_on_null(where, feature_count, dialect, ds_for_test_ogr_sql_on_null):
+
+    if feature_count is None:
+        if dialect == "SQLite":
+            with pytest.raises(Exception):
+                with ds_for_test_ogr_sql_on_null.ExecuteSQL(
+                    "select * from layer where " + where, dialect=dialect
+                ) as sql_lyr:
+                    pass
+        else:
+            with ds_for_test_ogr_sql_on_null.ExecuteSQL(
+                "select * from layer where " + where, dialect=dialect
+            ) as sql_lyr:
+                with pytest.raises(Exception):
+                    sql_lyr.GetFeatureCount()
+    else:
+        with ds_for_test_ogr_sql_on_null.ExecuteSQL(
+            "select * from layer where " + where, dialect=dialect
+        ) as sql_lyr:
+            assert sql_lyr.GetFeatureCount() == feature_count
+
+
+def test_ogr_sql_ogr_style_hidden():
+
+    ds = ogr.GetDriverByName("Memory").CreateDataSource("test_ogr_sql_ogr_style_hidden")
+    lyr = ds.CreateLayer("layer")
+    lyr.CreateField(ogr.FieldDefn("intfield", ogr.OFTInteger))
+    lyr.CreateField(ogr.FieldDefn("strfield", ogr.OFTString))
+    feat = ogr.Feature(lyr.GetLayerDefn())
+    feat["intfield"] = 1
+    feat["strfield"] = "my_style"
+    feat.SetGeometry(ogr.CreateGeometryFromWkt("POINT (1 2)"))
+    lyr.CreateFeature(feat)
+    feat = ogr.Feature(lyr.GetLayerDefn())
+    lyr.CreateFeature(feat)
+
+    with ds.ExecuteSQL(
+        "SELECT 'BRUSH(fc:#01234567)' AS OGR_STYLE HIDDEN FROM layer"
+    ) as sql_lyr:
+        assert sql_lyr.GetLayerDefn().GetFieldCount() == 0
+        f = sql_lyr.GetNextFeature()
+        assert f.GetStyleString() == "BRUSH(fc:#01234567)"
+
+    with ds.ExecuteSQL("SELECT strfield OGR_STYLE HIDDEN FROM layer") as sql_lyr:
+        assert sql_lyr.GetLayerDefn().GetFieldCount() == 0
+        f = sql_lyr.GetNextFeature()
+        assert f.GetStyleString() == "my_style"
+        f = sql_lyr.GetNextFeature()
+        assert f.GetStyleString() is None
+
+    with ds.ExecuteSQL(
+        "SELECT CAST(strfield AS CHARACTER(255)) AS OGR_STYLE HIDDEN FROM layer"
+    ) as sql_lyr:
+        assert sql_lyr.GetLayerDefn().GetFieldCount() == 0
+        f = sql_lyr.GetNextFeature()
+        assert f.GetStyleString() == "my_style"
+        f = sql_lyr.GetNextFeature()
+        assert f.GetStyleString() is None
+
+    with ds.ExecuteSQL("SELECT strfield OGR_STYLE HIDDEN, * FROM layer") as sql_lyr:
+        assert sql_lyr.GetLayerDefn().GetFieldCount() == 2
+        f = sql_lyr.GetNextFeature()
+        assert f.GetStyleString() == "my_style"
+        assert f["intfield"] == 1
+        assert f["strfield"] == "my_style"
+        f = sql_lyr.GetNextFeature()
+        assert f.GetStyleString() is None
+        assert not f.IsFieldSet("intfield")
+        assert not f.IsFieldSet("strfield")
+
+    with pytest.raises(
+        Exception, match="HIDDEN keyword only supported on a column named OGR_STYLE"
+    ):
+        with ds.ExecuteSQL(
+            "SELECT 'foo' AS not_OGR_STYLE HIDDEN FROM layer"
+        ) as sql_lyr:
+            pass
+
+    with ds.ExecuteSQL("SELECT 123 AS OGR_STYLE HIDDEN FROM layer") as sql_lyr:
+        gdal.ErrorReset()
+        with gdal.quiet_errors():
+            f = sql_lyr.GetNextFeature()
+        assert f.GetStyleString() is None
+
+    with ds.ExecuteSQL("SELECT intfield AS OGR_STYLE HIDDEN FROM layer") as sql_lyr:
+        gdal.ErrorReset()
+        with gdal.quiet_errors():
+            f = sql_lyr.GetNextFeature()
+        assert f.GetStyleString() is None
+
+    with ds.ExecuteSQL(
+        'SELECT "_ogr_geometry_" AS OGR_STYLE HIDDEN FROM layer'
+    ) as sql_lyr:
+        gdal.ErrorReset()
+        with gdal.quiet_errors():
+            f = sql_lyr.GetNextFeature()
+        assert f.GetStyleString() is None
+
+
+def test_ogr_sql_identifier_hidden():
+
+    ds = ogr.GetDriverByName("Memory").CreateDataSource("test_ogr_sql_ogr_style_hidden")
+    lyr = ds.CreateLayer("hidden")
+    lyr.CreateField(ogr.FieldDefn("hidden", ogr.OFTString))
+    feat = ogr.Feature(lyr.GetLayerDefn())
+    feat["hidden"] = "val"
+    lyr.CreateFeature(feat)
+
+    with ds.ExecuteSQL("SELECT hidden FROM hidden") as sql_lyr:
+        f = sql_lyr.GetNextFeature()
+        assert f["hidden"] == "val"
+
+    with ds.ExecuteSQL("SELECT hidden hidden FROM hidden hidden") as sql_lyr:
+        f = sql_lyr.GetNextFeature()
+        assert f["hidden"] == "val"
+
+    with ds.ExecuteSQL("SELECT hidden AS hidden FROM hidden AS hidden") as sql_lyr:
+        f = sql_lyr.GetNextFeature()
+        assert f["hidden"] == "val"
+
+    with ds.ExecuteSQL("SELECT 'foo' AS hidden FROM hidden") as sql_lyr:
+        f = sql_lyr.GetNextFeature()
+        assert f["hidden"] == "foo"
+
+
+@pytest.mark.parametrize(
+    "input,expected_output",
+    [
+        [(1, 1e100, 1, -1e100), 2],
+        [(float("inf"), 1), float("inf")],
+        [(1, float("-inf")), float("-inf")],
+        [(1, float("nan")), float("nan")],
+        [(float("inf"), float("-inf")), float("nan")],
+    ],
+)
+def test_ogr_sql_kahan_babuska_eumaier_summation(input, expected_output):
+    """Test accurate SUM() implementation using Kahan-Babuska-Neumaier algorithm"""
+
+    ds = ogr.GetDriverByName("Memory").CreateDataSource("")
+    lyr = ds.CreateLayer("test")
+    lyr.CreateField(ogr.FieldDefn("v", ogr.OFTReal))
+    for v in input:
+        feat = ogr.Feature(lyr.GetLayerDefn())
+        feat["v"] = v
+        lyr.CreateFeature(feat)
+
+    with ds.ExecuteSQL("SELECT SUM(v) FROM test") as sql_lyr:
+        f = sql_lyr.GetNextFeature()
+        if math.isnan(expected_output):
+            assert math.isnan(f["SUM_v"])
+        else:
+            assert f["SUM_v"] == expected_output
