@@ -12,25 +12,10 @@
 # Copyright (c) 2003, Frank Warmerdam <warmerdam@pobox.com>
 # Copyright (c) 2008-2013, Even Rouault <even dot rouault at spatialys.com>
 #
-# Permission is hereby granted, free of charge, to any person obtaining a
-# copy of this software and associated documentation files (the "Software"),
-# to deal in the Software without restriction, including without limitation
-# the rights to use, copy, modify, merge, publish, distribute, sublicense,
-# and/or sell copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included
-# in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-# DEALINGS IN THE SOFTWARE.
+# SPDX-License-Identifier: MIT
 ###############################################################################
 
+import json
 import os
 import subprocess
 import sys
@@ -1838,11 +1823,11 @@ def test_Set_PROJ_DATA_config_option():
 ###############################################################################
 
 
+@pytest.mark.skipif(
+    gdaltest.is_travis_branch("sanitize"), reason="fails on sanitize for unknown reason"
+)
 def test_Set_PROJ_DATA_config_option_sub_proccess_config_option_ok():
 
-    if gdaltest.is_travis_branch("sanitize"):
-        pytest.skip("fails on sanitize for unknown reason")
-
     backup_search_paths = osr.GetPROJSearchPaths()
     # conftest.py set 2 paths: autotest/gcore/tmp/proj_db_tmpdir and autotest/proj_grids
     assert len(backup_search_paths) == 2
@@ -1859,11 +1844,11 @@ def test_Set_PROJ_DATA_config_option_sub_proccess_config_option_ok():
 ###############################################################################
 
 
+@pytest.mark.skipif(
+    gdaltest.is_travis_branch("sanitize"), reason="fails on sanitize for unknown reason"
+)
 def test_Set_PROJ_DATA_config_option_sub_proccess_config_option_ko():
 
-    if gdaltest.is_travis_branch("sanitize"):
-        pytest.skip("fails on sanitize for unknown reason")
-
     backup_search_paths = osr.GetPROJSearchPaths()
     # conftest.py set 2 paths: autotest/gcore/tmp/proj_db_tmpdir and autotest/proj_grids
     assert len(backup_search_paths) == 2
@@ -2478,3 +2463,28 @@ def test_osr_basic_has_point_motion_operation():
     srs = osr.SpatialReference()
     srs.ImportFromEPSG(8255)  # NAD83(CSRS)v7
     assert srs.HasPointMotionOperation()
+
+
+###############################################################################
+
+
+# Test workaround for https://github.com/OSGeo/PROJ/pull/4166
+def test_osr_basic_export_wkt_utm_south():
+
+    srs = osr.SpatialReference()
+    srs.SetFromUserInput("+proj=utm +zone=1 +south +datum=WGS84")
+
+    assert 'ID["EPSG",16101]' in srs.ExportToWkt(["FORMAT=WKT2_2019"])
+
+    j = json.loads(srs.ExportToPROJJSON())
+    assert j["conversion"]["id"]["code"] == 16101
+
+
+###############################################################################
+
+
+def test_osr_basic_GetAuthorityListFromDatabase():
+
+    ret = osr.GetAuthorityListFromDatabase()
+    assert "EPSG" in ret
+    assert "PROJ" in ret
