@@ -13,23 +13,7 @@
 # Copyright (c) 2013, Kyle Shannon <kyle at pobox dot com>
 # Copyright (c) 2014, Google
 #
-# Permission is hereby granted, free of charge, to any person obtaining a
-# copy of this software and associated documentation files (the "Software"),
-# to deal in the Software without restriction, including without limitation
-# the rights to use, copy, modify, merge, publish, distribute, sublicense,
-# and/or sell copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included
-# in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-# DEALINGS IN THE SOFTWARE.
+# SPDX-License-Identifier: MIT
 ###############################################################################
 
 import pytest
@@ -1097,3 +1081,95 @@ def test_osr_esri_33():
 
 ###############################################################################
 #
+
+
+def test_osr_esri_lambert_azimutal_no_radius_of_sphere_of_reference():
+
+    prj = [
+        "Projection    LAMBERT_AZIMUTHAL",
+        "Datum         WGS84",
+        "Spheroid      WGS84",
+        "Units         METERS",
+        "Zunits        NO",
+        "Xshift        0.0",
+        "Yshift        0.0",
+        "Parameters",
+        "  20  0  0.0 /* longitude of center of projection",
+        "   5  0  0.0 /* latitude of center of projection",
+        "1.0 /* false easting (meters)",
+        "2.0 /* false northing (meters)",
+    ]
+
+    srs_prj = osr.SpatialReference()
+    srs_prj.ImportFromESRI(prj)
+
+    wkt = """PROJCS["unnamed",
+    GEOGCS["WGS 84",
+        DATUM["WGS_1984",
+            SPHEROID["WGS 84",6378137,298.257223563,
+                AUTHORITY["EPSG","7030"]],
+            AUTHORITY["EPSG","6326"]],
+        PRIMEM["Greenwich",0,
+            AUTHORITY["EPSG","8901"]],
+        UNIT["degree",0.0174532925199433,
+            AUTHORITY["EPSG","9122"]],
+        AUTHORITY["EPSG","4326"]],
+    PROJECTION["Lambert_Azimuthal_Equal_Area"],
+    PARAMETER["latitude_of_center",5],
+    PARAMETER["longitude_of_center",20],
+    PARAMETER["false_easting",1],
+    PARAMETER["false_northing",2],
+    UNIT["METERS",1],
+    AXIS["Easting",EAST],
+    AXIS["Northing",NORTH]]"""
+
+    srs_wkt = osr.SpatialReference(wkt=wkt)
+    assert srs_prj.IsSame(srs_wkt)
+
+
+###############################################################################
+#
+
+
+@pytest.mark.parametrize("has_semi_major_and_minor_axis", [True, False])
+def test_osr_esri_lambert_azimutal_radius_of_sphere_of_reference(
+    has_semi_major_and_minor_axis,
+):
+
+    prj = [
+        "Projection    LAMBERT_AZIMUTHAL",
+        "Units         METERS",
+        "Zunits        NO",
+        "Xshift        0.0",
+        "Yshift        0.0",
+        "Parameters    6378137.0  6378137.0"
+        if has_semi_major_and_minor_axis
+        else "Parameters",
+        "6378137.0 /* radius of the sphere of reference",
+        "  20  0  0.0 /* longitude of center of projection",
+        "   5  0  0.0 /* latitude of center of projection",
+        "1.0 /* false easting (meters)",
+        "2.0 /* false northing (meters)",
+    ]
+
+    srs_prj = osr.SpatialReference()
+    srs_prj.ImportFromESRI(prj)
+
+    wkt = """PROJCS["unnamed",
+    GEOGCS["unknown",
+        DATUM["unknown",
+            SPHEROID["unknown",6378137,0]],
+        PRIMEM["Greenwich",0],
+        UNIT["degree",0.0174532925199433,
+            AUTHORITY["EPSG","9122"]]],
+    PROJECTION["Lambert_Azimuthal_Equal_Area"],
+    PARAMETER["latitude_of_center",5],
+    PARAMETER["longitude_of_center",20],
+    PARAMETER["false_easting",1],
+    PARAMETER["false_northing",2],
+    UNIT["METERS",1],
+    AXIS["Easting",EAST],
+    AXIS["Northing",NORTH]]"""
+
+    srs_wkt = osr.SpatialReference(wkt=wkt)
+    assert srs_prj.IsSame(srs_wkt)
