@@ -12,23 +12,7 @@
 # Copyright (c) 2014, Etienne Tourigny <etourigny dot dev @ gmail dot com>
 # Copyright (c) 2020, Idan Miara <idan@miara.com>
 #
-# Permission is hereby granted, free of charge, to any person obtaining a
-# copy of this software and associated documentation files (the "Software"),
-# to deal in the Software without restriction, including without limitation
-# the rights to use, copy, modify, merge, publish, distribute, sublicense,
-# and/or sell copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included
-# in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-# DEALINGS IN THE SOFTWARE.
+# SPDX-License-Identifier: MIT
 ###############################################################################
 
 import os
@@ -36,7 +20,6 @@ import shutil
 from collections import defaultdict
 from copy import copy
 
-import gdaltest
 import pytest
 import test_py_scripts
 
@@ -70,9 +53,6 @@ def script_path():
 
 def test_gdal_calc_help(script_path):
 
-    if gdaltest.is_travis_branch("sanitize"):
-        pytest.skip("fails on sanitize for unknown reason")
-
     assert "ERROR" not in test_py_scripts.run_py_script(
         script_path, "gdal_calc", "--help"
     )
@@ -84,9 +64,6 @@ def test_gdal_calc_help(script_path):
 
 def test_gdal_calc_version(script_path):
 
-    if gdaltest.is_travis_branch("sanitize"):
-        pytest.skip("fails on sanitize for unknown reason")
-
     assert "ERROR" not in test_py_scripts.run_py_script(
         script_path, "gdal_calc", "--version"
     )
@@ -163,9 +140,14 @@ def test_gdal_calc_py_1(script_path, tmp_path, stefan_full_rgba):
     test_id, test_count = 1, 3
     out = make_temp_filename_list(tmp_path, test_id, test_count)
 
-    test_py_scripts.run_py_script(
-        script_path, "gdal_calc", f"-A {infile} --calc=A --overwrite --outfile {out[0]}"
+    _, err = test_py_scripts.run_py_script(
+        script_path,
+        "gdal_calc",
+        f"-A {infile} --calc=A --overwrite --outfile {out[0]}",
+        return_stderr=True,
     )
+    assert "UseExceptions" not in err
+
     test_py_scripts.run_py_script(
         script_path,
         "gdal_calc",
