@@ -9,38 +9,24 @@
 ###############################################################################
 # Copyright (c) 2017, Even Rouault <even dot rouault at spatialys dot com>
 #
-# Permission is hereby granted, free of charge, to any person obtaining a
-# copy of this software and associated documentation files (the "Software"),
-# to deal in the Software without restriction, including without limitation
-# the rights to use, copy, modify, merge, publish, distribute, sublicense,
-# and/or sell copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included
-# in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-# DEALINGS IN THE SOFTWARE.
+# SPDX-License-Identifier: MIT
 ###############################################################################
 
 import sys
 
-import gdaltest
 import pytest
 import test_py_scripts
 from test_py_scripts import samples_path
 
 from osgeo import gdal, ogr
 
-pytestmark = pytest.mark.skipif(
-    test_py_scripts.get_py_script("ogrmerge") is None,
-    reason="ogrmerge.py not available",
-)
+pytestmark = [
+    pytest.mark.require_driver("OGR_VRT"),
+    pytest.mark.skipif(
+        test_py_scripts.get_py_script("ogrmerge") is None,
+        reason="ogrmerge.py not available",
+    ),
+]
 
 
 @pytest.fixture()
@@ -78,7 +64,7 @@ def test_ogrmerge_1(script_path, tmp_path):
 
     out_shp = str(tmp_path / "out.shp")
 
-    test_py_scripts.run_py_script(
+    _, err = test_py_scripts.run_py_script(
         script_path,
         "ogrmerge",
         f"-single -o {out_shp} "
@@ -86,7 +72,9 @@ def test_ogrmerge_1(script_path, tmp_path):
         + "poly.shp "
         + test_py_scripts.get_data_path("ogr")
         + "poly.shp",
+        return_stderr=True,
     )
+    assert "UseExceptions" not in err
 
     ds = ogr.Open(out_shp)
     lyr = ds.GetLayer(0)
@@ -246,9 +234,6 @@ def test_ogrmerge_7(script_path, tmp_path):
 
     out_vrt = str(tmp_path / "out.vrt")
 
-    if gdaltest.is_travis_branch("sanitize"):
-        pytest.skip("fails on sanitize for unknown reason")
-
     # No match in -single mode
     test_py_scripts.run_py_script(
         script_path,
