@@ -10,23 +10,7 @@
 ###############################################################################
 # Copyright (c) 2008-2013, Even Rouault <even dot rouault at spatialys.com>
 #
-# Permission is hereby granted, free of charge, to any person obtaining a
-# copy of this software and associated documentation files (the "Software"),
-# to deal in the Software without restriction, including without limitation
-# the rights to use, copy, modify, merge, publish, distribute, sublicense,
-# and/or sell copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included
-# in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-# DEALINGS IN THE SOFTWARE.
+# SPDX-License-Identifier: MIT
 ###############################################################################
 
 import gdaltest
@@ -35,10 +19,16 @@ import test_cli_utilities
 
 from osgeo import gdal, osr
 
-pytestmark = pytest.mark.skipif(
-    test_cli_utilities.get_gdalbuildvrt_path() is None,
-    reason="gdalbuildvrt not available",
-)
+pytestmark = [
+    pytest.mark.skipif(
+        test_cli_utilities.get_gdalbuildvrt_path() is None,
+        reason="gdalbuildvrt not available",
+    ),
+    pytest.mark.skipif(
+        not gdaltest.vrt_has_open_support(),
+        reason="VRT driver open missing",
+    ),
+]
 
 
 @pytest.fixture(scope="module")
