@@ -10,23 +10,7 @@
 ###############################################################################
 # Copyright (c) 2009-2013, Even Rouault <even dot rouault at spatialys.com>
 #
-# Permission is hereby granted, free of charge, to any person obtaining a
-# copy of this software and associated documentation files (the "Software"),
-# to deal in the Software without restriction, including without limitation
-# the rights to use, copy, modify, merge, publish, distribute, sublicense,
-# and/or sell copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included
-# in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-# DEALINGS IN THE SOFTWARE.
+# SPDX-License-Identifier: MIT
 ###############################################################################
 
 import os
@@ -381,6 +365,10 @@ def test_gdaldem_color_relief_cpt(gdaldem_path, tmp_path):
 # Test gdaldem color relief to VRT
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_gdaldem_color_relief_vrt(gdaldem_path, n43_colorrelief_tif, tmp_path):
 
     output_vrt = str(tmp_path / "n43_colorrelief.vrt")
@@ -518,6 +506,10 @@ def test_gdaldem_color_relief_nearest_color_entry(gdaldem_path, tmp_path):
 # Test gdaldem color relief with -nearest_color_entry and -of VRT
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_gdaldem_color_relief_nearest_color_entry_vrt(gdaldem_path, tmp_path):
 
     output_vrt = str(tmp_path / "n43_colorrelief_nearest.vrt")
@@ -584,6 +576,10 @@ NODATA_value nan
 # Test gdaldem color relief with entries with repeated DEM values in the color table (#6422)
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 @pytest.mark.require_driver("AAIGRID")
 def test_gdaldem_color_relief_repeated_entry(gdaldem_path, tmp_path):
 
