@@ -10,26 +10,11 @@
 ###############################################################################
 # Copyright (c) 2019, Even Rouault <even.rouault at spatialys.com>
 #
-# Permission is hereby granted, free of charge, to any person obtaining a
-# copy of this software and associated documentation files (the "Software"),
-# to deal in the Software without restriction, including without limitation
-# the rights to use, copy, modify, merge, publish, distribute, sublicense,
-# and/or sell copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included
-# in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-# DEALINGS IN THE SOFTWARE.
+# SPDX-License-Identifier: MIT
 ###############################################################################
 
 import collections
+import os
 import pathlib
 import struct
 
@@ -41,6 +26,10 @@ from osgeo import gdal
 ###############################################################################
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_gdalmdimtranslate_no_arg(tmp_vsimem):
 
     tmpfile = tmp_vsimem / "out.vrt"
@@ -52,6 +41,10 @@ def test_gdalmdimtranslate_no_arg(tmp_vsimem):
 ###############################################################################
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_gdalmdimtranslate_multidim_to_mem():
 
     out_ds = gdal.MultiDimTranslate("", "data/mdim.vrt", format="MEM")
@@ -66,6 +59,10 @@ def test_gdalmdimtranslate_multidim_to_mem():
 ###############################################################################
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_gdalmdimtranslate_multidim_to_classic(tmp_vsimem):
 
     tmpfile = tmp_vsimem / "out.tif"
@@ -83,6 +80,10 @@ def test_gdalmdimtranslate_multidim_to_classic(tmp_vsimem):
 ###############################################################################
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_gdalmdimtranslate_multidim_1d_to_classic(tmp_vsimem):
 
     tmpfile = tmp_vsimem / "out.tif"
@@ -111,10 +112,16 @@ def test_gdalmdimtranslate_classic_to_classic(tmp_vsimem):
 ###############################################################################
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_gdalmdimtranslate_classic_to_multidim(tmp_vsimem):
 
     tmpfile = tmp_vsimem / "out.vrt"
     tmpgtifffile = tmp_vsimem / "tmp.tif"
+    if os.path.exists("../gcore/data/byte.tif.aux.xml"):
+        os.unlink("../gcore/data/byte.tif.aux.xml")
     ds = gdal.Translate(tmpgtifffile, "../gcore/data/byte.tif")
     ds.SetSpatialRef(None)
     ds = None
@@ -168,6 +175,10 @@ def test_gdalmdimtranslate_classic_to_multidim(tmp_vsimem):
 ###############################################################################
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_gdalmdimtranslate_array(tmp_vsimem):
 
     tmpfile = tmp_vsimem / "out.vrt"
@@ -249,6 +260,10 @@ def test_gdalmdimtranslate_array(tmp_vsimem):
 ###############################################################################
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_gdalmdimtranslate_array_with_transpose_and_view(tmp_vsimem):
 
     tmpfile = tmp_vsimem / "out.vrt"
@@ -324,6 +339,10 @@ def test_gdalmdimtranslate_array_with_transpose_and_view(tmp_vsimem):
 ###############################################################################
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_gdalmdimtranslate_group(tmp_vsimem):
 
     tmpfile = tmp_vsimem / "out.vrt"
@@ -389,6 +408,10 @@ def test_gdalmdimtranslate_group(tmp_vsimem):
 ###############################################################################
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_gdalmdimtranslate_two_groups(tmp_vsimem):
 
     tmpfile = tmp_vsimem / "out.vrt"
@@ -459,6 +482,10 @@ def test_gdalmdimtranslate_two_groups(tmp_vsimem):
 ###############################################################################
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_gdalmdimtranslate_subset(tmp_vsimem):
 
     tmpfile = tmp_vsimem / "out.vrt"
@@ -714,6 +741,10 @@ def test_gdalmdimtranslate_subset(tmp_vsimem):
 ###############################################################################
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_gdalmdimtranslate_scaleaxes(tmp_vsimem):
 
     tmpfile = tmp_vsimem / "out.vrt"
@@ -788,6 +819,10 @@ def test_gdalmdimtranslate_scaleaxes(tmp_vsimem):
     )
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_gdalmdimtranslate_dims_with_same_name_different_size(tmp_vsimem):
 
     srcfile = tmp_vsimem / "in.vrt"
